import matplotlib

matplotlib.use('Agg')

import matplotlib.pyplot as plt
import numpy as np
from matplotlib.ticker import MultipleLocator, FormatStrFormatter
from matplotlib import cm
from lib.common.fileIO import tsv_file_dict_parse
import matplotlib.gridspec as gridspec


def add_high(data, num):
    new_array = []
    for i in range(0, int(data.shape[0])):
        for j in range(0, num):
            new_array.append(data[i,])
    return np.array(new_array)


file = '/lustre/home/xuyuxing/Work/Other/Pseudouracil/all_tran/printdata.txt'

data_dir = tsv_file_dict_parse(file)

intersection_list = {"WT": ("S02_S01", "S06_S05", "S10_S26", "S25_S09"),
                     "sT": ("S04_S03", "S08_S07", "S12_S11", "S28_S27"),
                     "Wm": ("S18_S17", "S22_S30", "S29_S21", "S32_S31"),
                     "sm": ("S14_S13", "S16_S15", "S20_S19", "S24_S23")}

id = 'ATCG01170.1'

peak_data_WT = []
for peak_tab in intersection_list['WT']:
    peak_data_WT.append([float(data_dir[i][peak_tab]) for i in data_dir if data_dir[i]['Gene_id'] == id])

peak_data_sT = []
for peak_tab in intersection_list['sT']:
    peak_data_sT.append([float(data_dir[i][peak_tab]) for i in data_dir if data_dir[i]['Gene_id'] == id])

peak_data_WT = add_high(np.array(peak_data_WT), 3)
peak_data_sT = add_high(np.array(peak_data_sT), 3)

# plt.style.use('ggplot')

# Make plot with vertical (default) colorbar
# fig, (ax0, ax1) = plt.subplots(2, 1)

# plt.figure(figsize=(int(len(peak_data_WT[0]) / 100), 10))

gs = gridspec.GridSpec(5, 1, height_ratios=[3, 1, 1, 1, 1])

ax0 = plt.subplot(gs[0])
ax1 = plt.subplot(gs[1], sharex=ax0)
ax2 = plt.subplot(gs[2], sharex=ax0)
ax3 = plt.subplot(gs[3], sharex=ax0)
ax4 = plt.subplot(gs[4], sharex=ax0)

# plt.set_size_inches(18.5, 10.5)

# plt.subplots_adjust(wspace=0, hspace=-100)

cax0 = ax0.imshow(np.array(list(peak_data_WT) + list(peak_data_sT)), interpolation='nearest', cmap=cm.YlOrRd)
# plt.colorbar(cax0, ax=ax0)
# cax0 = ax0.imshow(peak_data_WT, interpolation='nearest', cmap=cm.YlOrRd)
# cax0 = ax0.imshow(peak_data_WT)
ax0.spines['top'].set_visible(False)
ax0.spines['right'].set_visible(False)
ax0.spines['bottom'].set_visible(False)
ax0.spines['left'].set_visible(False)
ax0.get_xaxis().set_visible(False)
ax0.set_xlim(0, len(peak_data_WT[0]))
line2 = ax0.axhline(y=11.5, color='w', linewidth=2)

WT_coverage_p = []
WT_coverage_n = []
for peak_tab in intersection_list['WT']:
    CMC_p, CMC_n = peak_tab.split('_')
    WT_coverage_p.append([float(data_dir[i][CMC_p]) for i in data_dir if data_dir[i]['Gene_id'] == id])
    WT_coverage_n.append([float(data_dir[i][CMC_n]) for i in data_dir if data_dir[i]['Gene_id'] == id])

WT_coverage_p = np.array(WT_coverage_p).mean(axis=0)

ax1.set_xlim(0, len(WT_coverage_p))

cax1 = ax1.plot(range(0, len(WT_coverage_p)), WT_coverage_p, 'b')
# cax1 = ax1.imshow(peak_data_sT)
ax1.spines['top'].set_visible(False)
ax1.spines['right'].set_visible(False)
ax1.spines['bottom'].set_visible(False)
ax1.spines['left'].set_visible(False)
ax1.get_xaxis().set_visible(False)

WT_coverage_n = np.array(WT_coverage_n).mean(axis=0)

ax2.set_xlim(0, len(WT_coverage_n))

cax2 = ax2.plot(range(0, len(WT_coverage_n)), WT_coverage_n, 'b--')
# cax1 = ax1.imshow(peak_data_sT)
ax2.spines['top'].set_visible(False)
ax2.spines['right'].set_visible(False)
ax2.spines['bottom'].set_visible(False)
ax2.spines['left'].set_visible(False)
ax2.get_xaxis().set_visible(False)

sT_coverage_p = []
sT_coverage_n = []
for peak_tab in intersection_list['sT']:
    CMC_p, CMC_n = peak_tab.split('_')
    sT_coverage_p.append([float(data_dir[i][CMC_p]) for i in data_dir if data_dir[i]['Gene_id'] == id])
    sT_coverage_n.append([float(data_dir[i][CMC_n]) for i in data_dir if data_dir[i]['Gene_id'] == id])

sT_coverage_p = np.array(sT_coverage_p).mean(axis=0)

ax3.set_xlim(0, len(sT_coverage_p))

cax3 = ax3.plot(range(0, len(sT_coverage_p)), sT_coverage_p, 'g')
# cax1 = ax1.imshow(peak_data_sT)
ax3.spines['top'].set_visible(False)
ax3.spines['right'].set_visible(False)
ax3.spines['bottom'].set_visible(False)
ax3.spines['left'].set_visible(False)
ax3.get_xaxis().set_visible(False)

sT_coverage_n = np.array(sT_coverage_n).mean(axis=0)

ax4.set_xlim(0, len(sT_coverage_p))

cax4 = ax4.plot(range(0, len(sT_coverage_n)), sT_coverage_n, 'g--')
# cax1 = ax1.imshow(peak_data_sT)
ax4.spines['top'].set_visible(False)
ax4.spines['right'].set_visible(False)
ax4.spines['bottom'].set_visible(False)
ax4.spines['left'].set_visible(False)

xmajorLocator = MultipleLocator(200)
ax0.xaxis.set_major_locator(xmajorLocator)
ax1.xaxis.set_major_locator(xmajorLocator)
ax2.xaxis.set_major_locator(xmajorLocator)
ax3.xaxis.set_major_locator(xmajorLocator)
ax4.xaxis.set_major_locator(xmajorLocator)
# plt.axis('off')
ax0.yaxis.set_major_locator(plt.NullLocator())
ax1.yaxis.set_major_locator(plt.NullLocator())
ax2.yaxis.set_major_locator(plt.NullLocator())
ax3.yaxis.set_major_locator(plt.NullLocator())
ax4.yaxis.set_major_locator(plt.NullLocator())

# plt.grid(False)
plt.savefig('/lustre/home/xuyuxing/Work/Other/Pseudouracil/all_tran/' + id + '.pdf', dpi=1000)
# fig.savefig('25S rRNA.pdf', dpi=1000)

plt.show()

##################################

import matplotlib

matplotlib.use('Agg')

import matplotlib.pyplot as plt
import numpy as np
from matplotlib.ticker import MultipleLocator, FormatStrFormatter
from matplotlib import cm
from lib.common.fileIO import tsv_file_dict_parse


def add_high(data, num):
    new_array = []
    for i in range(0, int(data.shape[0])):
        for j in range(0, num):
            new_array.append(data[i,])
    return np.array(new_array)


file = '/lustre/home/xuyuxing/Work/Other/Pseudouracil/all_tran/printdata.txt'

data_dir = tsv_file_dict_parse(file)

intersection_list = {"WT": ("S02_S01", "S06_S05", "S10_S26", "S25_S09"),
                     "sT": ("S04_S03", "S08_S07", "S12_S11", "S28_S27"),
                     "Wm": ("S18_S17", "S22_S30", "S29_S21", "S32_S31"),
                     "sm": ("S14_S13", "S16_S15", "S20_S19", "S24_S23")}

id = 'ATCG01180.1'

peak_data_WT = []
for peak_tab in intersection_list['WT']:
    peak_data_WT.append([float(data_dir[i][peak_tab]) for i in data_dir if data_dir[i]['Gene_id'] == id])

peak_data_sT = []
for peak_tab in intersection_list['sT']:
    peak_data_sT.append([float(data_dir[i][peak_tab]) for i in data_dir if data_dir[i]['Gene_id'] == id])

peak_data_WT = add_high(np.array(peak_data_WT), 10)
peak_data_sT = add_high(np.array(peak_data_sT), 10)

# plt.style.use('ggplot')

# Make plot with vertical (default) colorbar
fig, (ax0, ax1) = plt.subplots(2, 1)

fig.set_size_inches(18.5, 10.5)

plt.subplots_adjust(wspace=0, hspace=0)

cax0 = ax0.imshow(np.array(list(peak_data_WT) + list(peak_data_sT)), interpolation='nearest', cmap=cm.YlOrRd)
# fig.colorbar(cax0, ax=ax0)
# cax0 = ax0.imshow(peak_data_WT, interpolation='nearest', cmap=cm.YlOrRd)
# cax0 = ax0.imshow(peak_data_WT)

WT_coverage = []
for peak_tab in intersection_list['WT']:
    CMC_p, CMC_n = peak_tab.split('_')
    WT_coverage.append([float(data_dir[i][CMC_p]) for i in data_dir if data_dir[i]['Gene_id'] == id])
    WT_coverage.append([float(data_dir[i][CMC_n]) for i in data_dir if data_dir[i]['Gene_id'] == id])

cax1 = ax1.plot(range(0, len(WT_coverage[0])), WT_coverage[0])
# cax1 = ax1.imshow(peak_data_sT)

xmajorLocator = MultipleLocator(200)
ax0.xaxis.set_major_locator(xmajorLocator)
ax1.xaxis.set_major_locator(xmajorLocator)
plt.axis('off')
ax0.yaxis.set_major_locator(plt.NullLocator())
ax1.yaxis.set_major_locator(plt.NullLocator())

plt.grid(False)
fig.savefig('/lustre/home/xuyuxing/Work/Other/Pseudouracil/all_tran/' + id + '.pdf', dpi=1000)
# fig.savefig('25S rRNA.pdf', dpi=1000)

plt.show()


def add_high(data, num):
    new_array = []
    for i in range(0, int(data.shape[0])):
        for j in range(0, num):
            new_array.append(data[i,])
    return np.array(new_array)


plt.style.use('ggplot')
from numpy.random import randn

# Make plot with vertical (default) colorbar
fig, ax = plt.subplots()
fig.set_size_inches(18.5, 10.5)
# data = np.clip(randn(250, 250), -1, 1)

data = np.loadtxt("/lustre/home/xuyuxing/Work/Other/tmp/printdata.txt", delimiter="\t")

# 18s rRNA
data = data.transpose()[:, 0:2075]
# 25s rRNA
# data = data.transpose()[:, 2428:5902]
# data = data.transpose()

maker = data[0,]
other_data = np.array([data[1,], data[2,], data[3,], data[4,]])
other_data2 = np.array([data[5,], data[6,], data[7,], data[8,]])

data2 = add_high(other_data, 10)
data3 = add_high(other_data2, 10)
# data = np.array([[1,2],[3,4],[5,6]])

data2 = np.array(
    list(data2) + list(data3) + [maker, maker, maker, maker, maker, maker, maker, maker, maker, maker, maker])

cax = ax.imshow(data2, interpolation='nearest', cmap=cm.YlOrRd)
# ax.set_title('Gaussian noise with vertical colorbar')

# Add colorbar, make sure to specify tick locations to match desired ticklabels
# cbar = fig.colorbar(cax, ticks=[-1, 0, 1])
# cbar.ax.set_yticklabels(['< -1', '0', '> 1'])  # vertically oriented colorbar

line = ax.axhline(y=79, color='w', linewidth=0.8)
line2 = ax.axhline(y=39, color='w', linewidth=0.8)

xmajorLocator = MultipleLocator(200)
ax.xaxis.set_major_locator(xmajorLocator)

# plt.axis('off')
ax.yaxis.set_major_locator(plt.NullLocator())
plt.grid(False)
fig.savefig('18S rRNA.pdf', dpi=1000)
# fig.savefig('25S rRNA.pdf', dpi=1000)

plt.show()



if __name__ == '__main__':
    import argparse

    # argument parse
    # ---------------------------------------------------------
    parser = argparse.ArgumentParser(
        prog='GenFeatAnno',
    )

    subparsers = parser.add_subparsers(
        title='subcommands', dest="subcommand_name")

    # argparse for Telomere
    parser_a = subparsers.add_parser('Telomere',
                                     help='find telomeres in an assembly\n',
                                     description='')

    parser_a.add_argument('fasta_file', type=str, help='a fasta file')
    parser_a.add_argument('-d', '--dat', type=str,
                          help='if you already have a data file, you can input it with -d', default=None)
    parser_a.add_argument('-trf', '--trf_path', type=str,
                          help='path for trf program (default: trf)', default="trf")
    parser_a.add_argument('-l', '--log_file', type=str,
                          help='path for log file (default: None)', default=None)
    parser_a.add_argument('-o', '--output_prefix', type=str,
                          help='output prefix (default: output_)', default='output_')

    # argparse for Centromere
    parser_a = subparsers.add_parser('Centromere',
                                     help='find centromeres in an assembly\n',
                                     description='')

    parser_a.add_argument('-f', '--fasta_file', type=str, help='a fasta file')
    parser_a.add_argument('-d', '--trf_dat', type=str,
                          help='if you already have a data file, you can input it with -d', default=None)
    parser_a.add_argument('-o', '--work_dir', type=str,
                          help='output dir (default: centromeres)', default='centromeres')
    parser_a.add_argument("-t", "--threads", help="num of threads (default:56)", default=56, type=int)                          

    # argparse for rRNAFinder
    parser_a = subparsers.add_parser('rRNAFinder',
                                     help='find rRNA unit and all rRNA in genome\n',
                                     description='')

    parser_a.add_argument('fasta_file', type=str, help='a fasta file')
    parser_a.add_argument('-s', '--query_rRNA_seq', type=str, default=None,
                          help='a known rRNA unit seq from close species')
    parser_a.add_argument('-b', '--query_rRNA_bed', type=str, default=None,
                          help='bed file for close rRNA unit seq')
    parser_a.add_argument('-l', '--log_file', type=str,
                          help='path for log file (default: None)', default=None)
    parser_a.add_argument('-o', '--output_dir', type=str, help='output prefix (default: rRNA_finder)',
                          default='rRNA_finder')

    args = parser.parse_args()
    args_dict = vars(args)

    # ---------------------------------------------------------
    # command detail

    # Telomere
    if args_dict["subcommand_name"] == "Telomere":
        from src.xuyuxing.GenomeTools.Telomere import Telomere_main
        Telomere_main(args)

    # Centromere
    elif args_dict["subcommand_name"] == "Centromere":
        from src.xuyuxing.GenomeTools.CentromereSeed import CentromereSeed_main
        CentromereSeed_main(args)

    # rRNAFinder
    elif args_dict["subcommand_name"] == "rRNAFinder":
        from src.xuyuxing.GenomeTools.rRNAFinder import rRNAFinder_main
        rRNAFinder_main(args)



from lib.common.os import mkdir, copy_file, cmd_run, multiprocess_running
from lib.common.genome.seq_base import read_fasta_by_faidx
from lib.common.util import logging_init
from lib.common.evolution.orthotools2 import OrthoGroups, get_orthogroups_from_tree, get_non_overlap_orthogroups, OGs_gene_rename, merge_OGs, read_species_info, if_conserved
from Bio import Phylo
from lib.common.fileIO import tsv_file_dict_parse
from lib.common.evolution.tree_operate import get_root_by_species, reroot_by_outgroup_clade, add_clade_name, lookup_by_names


def prepare_OG_work_dir(OGs, species_info_dict, top_work_dir, need_cds_flag=False):
    mkdir(top_work_dir, True)

    huge_pt_dict = {}
    if need_cds_flag:
        huge_cds_dict = {}
    huge_rename_map = {}

    # read fasta
    for sp_id in OGs.species_list:
        sp = species_info_dict[sp_id]
        pt_fa_dict = read_fasta_by_faidx(sp.pt_file)
        if need_cds_flag:
            cds_fa_dict = read_fasta_by_faidx(sp.cds_file)

        pt_dict = {}
        if need_cds_flag:
            cds_dict = {}
        rename_map = {}

        num = 0
        for j in pt_fa_dict:
            pt_dict[num] = pt_fa_dict[j].seq
            if need_cds_flag:
                cds_dict[num] = cds_fa_dict[j].seq
            rename_map[j] = num
            num += 1

        huge_pt_dict[sp_id] = pt_dict
        if need_cds_flag:
            huge_cds_dict[sp_id] = cds_dict
        huge_rename_map[sp_id] = rename_map

    # write seq to each OG
    for OG_id in OGs.OG_dict:
        OG_work = top_work_dir + "/" + OG_id
        mkdir(OG_work, True)

        # pt_file
        pt_fa = OG_work + "/pt.fa"
        with open(pt_fa, 'w') as f:
            OG_gene_dict = OGs.get((OG_id, None))
            for sp_id in OG_gene_dict:
                for gene in OG_gene_dict[sp_id]:
                    g_id = gene.id
                    if g_id != '':
                        pt_seq = huge_pt_dict[sp_id][huge_rename_map[sp_id][g_id]]
                        f.write(">%s\n%s\n" % (
                            str(huge_rename_map[sp_id][g_id])+"_"+sp_id, pt_seq))

        # cds_file
        if need_cds_flag:
            cds_fa = OG_work + "/cds.fa"
            with open(cds_fa, 'w') as f:
                for sp_id in OG_gene_dict:
                    for gene in OG_gene_dict[sp_id]:
                        g_id = gene.id
                        if g_id != '':
                            cds_seq = huge_cds_dict[sp_id][huge_rename_map[sp_id][g_id]]
                            f.write(">%s\n%s\n" % (
                                str(huge_rename_map[sp_id][g_id])+"_"+sp_id, cds_seq))

        # rename_map
        rename_map = OG_work + "/rename.map"
        with open(rename_map, 'w') as f:
            for sp_id in OG_gene_dict:
                for gene in OG_gene_dict[sp_id]:
                    g_id = gene.id
                    if g_id != '':
                        f.write("%s\t%s\t%s\n" %
                                (huge_rename_map[sp_id][g_id], g_id, sp_id))

    # write rename file
    all_rename_map = top_work_dir + "/rename.map"
    with open(all_rename_map, 'w') as f:
        for sp_id in huge_rename_map:
            for g_id in huge_rename_map[sp_id]:
                if g_id != '':
                    f.write("%s\t%s\t%s\n" %
                            (huge_rename_map[sp_id][g_id], g_id, sp_id))


def fasttree_one(OG_work_dir):
    pt_fa = OG_work_dir + "/pt.fa"

    aa_aln_file = pt_fa+".aln"
    aa_dnd_file = pt_fa+".dnd"
    #cmd_string = "t_coffee "+aa_file+" -method mafftgins_msa muscle_msa kalign_msa t_coffee_msa -output=fasta_aln -outfile="+aa_aln_file+" -newtree "+aa_dnd_file
    cmd_string = "clustalw2 -INFILE="+pt_fa + \
        " -ALIGN -OUTPUT=FASTA -OUTFILE="+aa_aln_file+" -type=protein"
    cmd_run(cmd_string, silence=True)

    tree_file = OG_work_dir+"/fasttree.phb"
    cmd_string = "FastTree -wag -gamma -out %s %s >/dev/null" % (
        tree_file, aa_aln_file)
    cmd_run(cmd_string, silence=True)

    return pt_fa, aa_aln_file, tree_file


def fasttree_many(OGs, species_info_dict, work_dir, num_threads):
    mkdir(work_dir, True)
    log_file = work_dir + "/log"

    module_log = logging_init("FastTree pipeline", log_file)

    # load orthofinder group
    module_log.info('load orthofinder group')

    # prepare OG work dir
    module_log.info('prepare OG work dir')
    prepare_OG_work_dir(OGs, species_info_dict, work_dir)

    # running treebest
    module_log.info('running fasttree')
    args_list = []

    for OG_id in OGs.OG_dict:
        OG_work_dir = work_dir + "/" + OG_id
        args_list.append((OG_work_dir,))

    multiprocess_running(fasttree_one, args_list,
                         num_threads, log_file=log_file)

    return work_dir


def read_rename_map_file(rename_map_file):
    info_dict = tsv_file_dict_parse(rename_map_file, fieldnames=[
                                    "new_id", "old_id", "sp_id"])
    gene_to_species_map_dict = {}
    rename_dict = {}
    for i in info_dict:
        gene_to_species_map_dict[info_dict[i]["new_id"] +
                                 "_" + info_dict[i]["sp_id"]] = info_dict[i]["sp_id"]
        rename_dict[info_dict[i]["new_id"] + "_" +
                    info_dict[i]["sp_id"]] = info_dict[i]["old_id"]

    return rename_dict, gene_to_species_map_dict


unrooted_tree_file = '/lustre/home/xuyuxing/Work/Gel/synteny/20210311/fasttree/OG0001487/fasttree.phb'
species_tree_file = '/lustre/home/xuyuxing/Work/Gel/synteny/20210311/species.tree.txt'
rename_map_file = '/lustre/home/xuyuxing/Work/Gel/synteny/20210311/fasttree/OG0001487/rename.map'
output_tree_file = '/lustre/home/xuyuxing/Work/Gel/synteny/20210311/fasttree/OG0001487/fasttree.rooted.phb'

def rooted_tree(unrooted_tree_file, species_tree_file, rename_map_file, output_tree_file):
    species_tree = Phylo.read(species_tree_file, 'newick')
    unrooted_gene_tree = Phylo.read(unrooted_tree_file, 'newick')
    unrooted_gene_tree = add_clade_name(unrooted_gene_tree)
    unrooted_gene_tree_node_dict = lookup_by_names(unrooted_gene_tree)

    rename_dict, gene_to_species_map_dict = read_rename_map_file(rename_map_file)

    best_root_clade = get_root_by_species(unrooted_gene_tree, species_tree, gene_to_species_map_dict)

    gene_tree_rooted, gene_tree_rooted_node_dict, gene_tree, gene_tree_node_dict = reroot_by_outgroup_clade(unrooted_gene_tree,
                                                                                                            unrooted_gene_tree_node_dict,
                                                                                                            best_root_clade.name,
                                                                                                            True)

    for i in gene_tree_rooted_node_dict:
        c = gene_tree_rooted_node_dict[i]
        if not c.is_terminal():
            c.name = None

    Phylo.write(gene_tree_rooted, output_tree_file, format='newick')


def get_suborthogroups_from_one_tree_dir(tree_prefix, gene_tree_file, species_tree_file, gene_to_species_map_file, conserved_function, conserved_arguments, support_threshold, no_overlap=True):

    gene_tree = Phylo.read(gene_tree_file, 'newick')
    species_tree = Phylo.read(species_tree_file, 'newick')

    info_dict = tsv_file_dict_parse(gene_to_species_map_file, fieldnames=[
                                    "new_id", "old_id", "sp_id"])
    gene_to_species_map_dict = {}
    rename_dict = {}
    for i in info_dict:
        gene_to_species_map_dict[info_dict[i]["new_id"] +
                                 "_" + info_dict[i]["sp_id"]] = info_dict[i]["sp_id"]
        rename_dict[info_dict[i]["new_id"] + "_" +
                    info_dict[i]["sp_id"]] = info_dict[i]["old_id"]

    sp_in_tree = list(set([gene_to_species_map_dict[i.name]
                           for i in gene_tree.get_terminals()]))
    gene_in_tree = list(set([i.name for i in gene_tree.get_terminals()]))
    sp_list = list(set([i.name for i in species_tree.get_terminals()]))

    if conserved_function(sp_in_tree, conserved_arguments):

        OGs = get_orthogroups_from_tree(tree_prefix, gene_tree, species_tree,
                                        gene_to_species_map_dict, conserved_function, conserved_arguments, support_threshold)

        if no_overlap:
            OGs = get_non_overlap_orthogroups(OGs)

        OGs = OGs_gene_rename(OGs, rename_dict)

    else:
        OG_dict = {tree_prefix: {i: [] for i in sp_list}}
        for g_id in gene_in_tree:
            OG_dict[tree_prefix][gene_to_species_map_dict[g_id]].append(g_id)

        OGs = OrthoGroups(from_OG_dict=OG_dict, species_list=sp_list)
        OGs = OGs_gene_rename(OGs, rename_dict)

    return OGs


def get_orthogroups_from_fasttree_pipeline(OG_tsv_file, fasttree_dir, species_tree_file, conserved_function, conserved_arguments, support_threshold, threads=56, no_overlap=True):

    OGs = OrthoGroups(OG_tsv_file=OG_tsv_file)

    num = 0

    args_list = []

    for og_id in OGs.OG_id_list:
        num += 1

        og_dir = fasttree_dir + "/" + og_id
        gene_tree_file = og_dir + "/fasttree.phb"
        gene_to_species_map_file = og_dir + "/rename.map"
        args_list.append((og_id, gene_tree_file, species_tree_file, gene_to_species_map_file,
                          conserved_function, conserved_arguments, support_threshold, no_overlap))

    tmp_out = multiprocess_running(
        get_suborthogroups_from_one_tree_dir, args_list, threads, None, True, None)

    OGs_list = [tmp_out[i]['output'] for i in tmp_out]

    OGs = merge_OGs(OGs_list)

    return OGs


if __name__ == '__main__':

    # running fasttree pipeline

    ref_xlsx = '/lustre/home/xuyuxing/Work/Other/Canrong/orthofinder/seq_info.xlsx'
    sp_list = ["Osa", "Zma", "Acom", "Xvi",
               "Aco", "Ath", "Mdo", "Sly", "Pni", "Atr"]
    OG_tsv_file = "/lustre/home/xuyuxing/Work/Other/Canrong/orthofinder/pt_file/OrthoFinder/Results_May08/Orthogroups/Orthogroups.tsv"
    fasttree_dir = "/lustre/home/xuyuxing/Work/Other/Canrong/orthofinder/fasttree"

    OGs = OrthoGroups(OG_tsv_file=OG_tsv_file)

    species_info_dict = read_species_info(ref_xlsx)
    species_info_dict = {i: species_info_dict[i]
                         for i in species_info_dict if i in sp_list}

    fasttree_many(OGs, species_info_dict, fasttree_dir, 56)

    # get orthogroups
    OG_tsv_file = "/lustre/home/xuyuxing/Work/Other/Canrong/orthofinder/pt_file/OrthoFinder/Results_May08/Orthogroups/Orthogroups.tsv"
    fasttree_dir = "/lustre/home/xuyuxing/Work/Other/Canrong/orthofinder/fasttree"
    species_tree_file = "/lustre/home/xuyuxing/Work/Other/Canrong/orthofinder/species.txt"
    conserved_arguments = [
        [["Osa", "Zma", "Acom", "Xvi"], ["Aco", "Ath", "Mdo", "Sly"]], [2, 2]]
    support_threshold = 0.5
    no_overlap_OGs = get_orthogroups_from_fasttree_pipeline(
        OG_tsv_file, fasttree_dir, species_tree_file, if_conserved, conserved_arguments, support_threshold, threads=56, no_overlap=True)

    overlap_OGs = get_orthogroups_from_fasttree_pipeline(
        OG_tsv_file, fasttree_dir, species_tree_file, if_conserved, conserved_arguments, support_threshold, threads=56, no_overlap=False)

    # write
    no_overlap_OGs.write_OG_tsv_file(
        "/lustre/home/xuyuxing/Work/Other/Canrong/orthofinder/fasttree/no_overlap_OGs.tsv")
    overlap_OGs.write_OG_tsv_file(
        "/lustre/home/xuyuxing/Work/Other/Canrong/orthofinder/fasttree/overlap_OGs.tsv")



step_1_3_shell = """#! /bin/bash

# http://weatherby.genetics.utah.edu/MAKER/wiki/index.php/Repeat_Library_Construction-Advanced

source activate maker_p

GENOME=$1
TAG=$2
tRNA_file=%s
CRL_DIR=%s

# 2.1.1. Collection of candidate elements with LTRs that are 99%% or more in similarity using LTRharvest
mkdir $TAG
cd $TAG

gt suffixerator -db $GENOME -indexname $TAG.index -tis -suf -lcp -des -ssp -dna
gt ltrharvest -index $TAG.index -out $TAG.out99 -outinner $TAG.outinner99 -gff3 $TAG.gff99 -minlenltr 100 -maxlenltr 6000 -mindistltr 1500 -maxdistltr 25000 -mintsd 5 -maxtsd 5 -motif tgca -similar 99 -vic 10  > $TAG.result99

# 2.1.2. Using LTRdigest to find elements with PPT (poly purine tract) or PBS (primer binding site)
gt gff3 -sort $TAG.gff99 > $TAG.gff99.sort
gt ltrdigest -trnas $tRNA_file $TAG.gff99.sort $TAG.index > $TAG.gff99.dgt

perl $CRL_DIR/CRL_Step1.pl --gff $TAG.gff99.dgt

# 2.1.3. Further filtering of the candidate elements
perl $CRL_DIR/CRL_Step2.pl --step1 CRL_Step1_Passed_Elements.txt --repeatfile $TAG.out99 --resultfile $TAG.result99 --sequencefile $GENOME --removed_repeats CRL_Step2_Passed_Elements.fasta

mkdir fasta_files
mv Repeat_*.fasta fasta_files
mv CRL_Step2_Passed_Elements.fasta fasta_files
cd fasta_files

perl $CRL_DIR/CRL_Step3.pl --directory ../fasta_files --step2 CRL_Step2_Passed_Elements.fasta --pidentity 60 --seq_c 25

mv  CRL_Step3_Passed_Elements.fasta  ..
cd  ..
        
"""

step_1_3_85_shell = """#! /bin/bash

# http://weatherby.genetics.utah.edu/MAKER/wiki/index.php/Repeat_Library_Construction-Advanced
# 2.2. Collection of relatively old LTR retrotransposons

source activate maker_p

GENOME=$1
TAG=$2
tRNA_file=%s
CRL_DIR=%s

# 2.1.1
mkdir ${TAG}
cd ${TAG}

gt suffixerator -db $GENOME -indexname $TAG.index -tis -suf -lcp -des -ssp -dna
gt ltrharvest -index $TAG.index -out $TAG.out85 -outinner $TAG.outinner85 -gff3 $TAG.gff85 -minlenltr 100 -maxlenltr 6000 -mindistltr 1500 -maxdistltr 25000 -mintsd 5 -maxtsd 5 -vic 10  > $TAG.result85

# 2.1.2. Using LTRdigest to find elements with PPT (poly purine tract) or PBS (primer binding site)
gt gff3 -sort $TAG.gff85 > $TAG.gff85.sort
gt ltrdigest -trnas $tRNA_file $TAG.gff85.sort $TAG.index > $TAG.gff85.dgt

perl $CRL_DIR/CRL_Step1.pl --gff $TAG.gff85.dgt

# 2.1.3. Further filtering of the candidate elements
perl $CRL_DIR/CRL_Step2.pl --step1 CRL_Step1_Passed_Elements.txt --repeatfile $TAG.out85 --resultfile $TAG.result85 --sequencefile $GENOME --removed_repeats CRL_Step2_Passed_Elements.fasta

mkdir fasta_files
mv Repeat_*.fasta fasta_files
mv CRL_Step2_Passed_Elements.fasta fasta_files
cd fasta_files

perl $CRL_DIR/CRL_Step3.pl --directory ../fasta_files --step2 CRL_Step2_Passed_Elements.fasta --pidentity 60 --seq_c 25

mv  CRL_Step3_Passed_Elements.fasta  ..
cd  ..

"""

step_4_5_shell = """#! /bin/bash

# http://weatherby.genetics.utah.edu/MAKER/wiki/index.php/Repeat_Library_Construction-Advanced

source activate maker_p

GENOME=%s
TAG=all
tRNA_file=%s
CRL_DIR=%s
MITE_file=%s
Tpase_dir=%s

# 2.1.4. Identify elements with nested insertions

perl $CRL_DIR/ltr_library.pl --resultfile $TAG.result99 --step3 $TAG.CRL_Step3_Passed_Elements.fasta --sequencefile $GENOME
cat lLTR_Only.lib $MITE_file  > repeats_to_mask_LTR99.fasta
RepeatMasker -pa 56 -lib repeats_to_mask_LTR99.fasta -nolow -dir . $TAG.outinner99
perl $CRL_DIR/cleanRM.pl $TAG.outinner99.out $TAG.outinner99.masked > $TAG.outinner99.unmasked
perl $CRL_DIR/rmshortinner.pl $TAG.outinner99.unmasked 50 > $TAG.outinner99.clean

blastx -query $TAG.outinner99.clean -db $Tpase_dir/Tpases020812DNA -evalue 1e-10 -num_descriptions 10 -out $TAG.outinner99.clean_blastx.out.txt -num_threads 56
perl $CRL_DIR/outinner_blastx_parse.pl --blastx $TAG.outinner99.clean_blastx.out.txt --outinner $TAG.outinner99

# 2.1.5 Building examplars

perl $CRL_DIR/CRL_Step4.pl --step3 $TAG.CRL_Step3_Passed_Elements.fasta --resultfile $TAG.result99 --innerfile passed_outinner_sequence.fasta --sequencefile $GENOME

makeblastdb -in lLTRs_Seq_For_BLAST.fasta -dbtype nucl
blastn -query lLTRs_Seq_For_BLAST.fasta -db lLTRs_Seq_For_BLAST.fasta -evalue 1e-10 -num_descriptions 1000 -out lLTRs_Seq_For_BLAST.fasta.out -num_threads 56

makeblastdb -in Inner_Seq_For_BLAST.fasta -dbtype nucl
blastn -query Inner_Seq_For_BLAST.fasta -db Inner_Seq_For_BLAST.fasta  -evalue 1e-10 -num_descriptions 1000 -out  Inner_Seq_For_BLAST.fasta.out -num_threads 56

perl $CRL_DIR/CRL_Step5.pl --LTR_blast lLTRs_Seq_For_BLAST.fasta.out --inner_blast Inner_Seq_For_BLAST.fasta.out --step3 $TAG.CRL_Step3_Passed_Elements.fasta --final LTR99.lib --pcoverage 90 --pidentity 80

"""


step_4_5_85_shell = """#! /bin/bash

# http://weatherby.genetics.utah.edu/MAKER/wiki/index.php/Repeat_Library_Construction-Advanced

source activate maker_p

GENOME=%s
TAG=all
tRNA_file=%s
CRL_DIR=%s
MITE_file=%s
Tpase_dir=%s

# 2.1.4. Identify elements with nested insertions

perl $CRL_DIR/ltr_library.pl --resultfile $TAG.result85 --step3 $TAG.CRL_Step3_Passed_Elements.fasta --sequencefile $GENOME
cat lLTR_Only.lib $MITE_file  > repeats_to_mask_LTR85.fasta
RepeatMasker -pa 56 -lib repeats_to_mask_LTR85.fasta -nolow -dir . $TAG.outinner85
perl $CRL_DIR/cleanRM.pl $TAG.outinner85.out $TAG.outinner85.masked > $TAG.outinner85.unmasked
perl $CRL_DIR/rmshortinner.pl $TAG.outinner85.unmasked 50 > $TAG.outinner85.clean

blastx -query $TAG.outinner85.clean -db $Tpase_dir/Tpases020812DNA -evalue 1e-10 -num_descriptions 10 -out $TAG.outinner85.clean_blastx.out.txt -num_threads 56
perl $CRL_DIR/outinner_blastx_parse.pl --blastx $TAG.outinner85.clean_blastx.out.txt --outinner $TAG.outinner85

# 2.1.5 Building examplars

perl $CRL_DIR/CRL_Step4.pl --step3 $TAG.CRL_Step3_Passed_Elements.fasta --resultfile $TAG.result85 --innerfile passed_outinner_sequence.fasta --sequencefile $GENOME

makeblastdb -in lLTRs_Seq_For_BLAST.fasta -dbtype nucl
blastn -query lLTRs_Seq_For_BLAST.fasta -db lLTRs_Seq_For_BLAST.fasta -evalue 1e-10 -num_descriptions 1000 -out lLTRs_Seq_For_BLAST.fasta.out -num_threads 56

makeblastdb -in Inner_Seq_For_BLAST.fasta -dbtype nucl
blastn -query Inner_Seq_For_BLAST.fasta -db Inner_Seq_For_BLAST.fasta  -evalue 1e-10 -num_descriptions 1000 -out  Inner_Seq_For_BLAST.fasta.out -num_threads 56

perl $CRL_DIR/CRL_Step5.pl --LTR_blast lLTRs_Seq_For_BLAST.fasta.out --inner_blast Inner_Seq_For_BLAST.fasta.out --step3 $TAG.CRL_Step3_Passed_Elements.fasta --final LTR85.lib --pcoverage 90 --pidentity 80

"""

MITE_shell = """source activate maker_p && \\
cp %s %s/genome.fasta && \\
perl %s -i genome.fasta -g %s -c %d -n %d -S 12345678 && \\
cat *Step8_* > MITE.lib
"""

merge85_99_shell = """source activate maker_p && \\
RepeatMasker -pa 56 -lib LTR99.lib -dir . LTR85.lib && \\
perl %s/remove_masked_sequence.pl --masked_elements LTR85.lib.masked --outfile FinalLTR85.lib && \\
cat LTR99.lib FinalLTR85.lib > allLTR.lib
"""

RepeatMasker_shell = """#! /bin/bash
    
source activate maker_p

GENOME=$1
TAG=$2
all_LIB=%s

mkdir ${TAG}
cd ${TAG}

RepeatMasker -lib $all_LIB -dir . $GENOME
"""

RepeatModeler_shell = """#! /bin/bash
CRL_DIR=%s
Tpase_dir=%s
            
source activate maker_p

cat */*.fa.masked > all.fa.masked
perl $CRL_DIR/rmaskedpart.pl all.fa.masked 50  >  all.fa.umseqfile
sed '/^$/d' all.fa.umseqfile > all.fa.fullline.umseqfile
BuildDatabase -name umseqfiledb -engine ncbi all.fa.fullline.umseqfile
RepeatModeler -pa 14 -database umseqfiledb

perl $CRL_DIR/repeatmodeler_parse.pl --fastafile RM*/consensi.fa.classified --unknowns repeatmodeler_unknowns.fasta --identities repeatmodeler_identities.fasta

# makeblastdb -in $Tpase_dir/Tpases020812  -dbtype prot
blastx -query repeatmodeler_unknowns.fasta -db $Tpase_dir/Tpases020812 -num_threads 56 -evalue 1e-10 -num_descriptions 10 -out modelerunknown_blast_results.txt
perl $CRL_DIR/transposon_blast_parse.pl --blastx modelerunknown_blast_results.txt --modelerunknown repeatmodeler_unknowns.fasta

mv  unknown_elements.txt  ModelerUnknown.lib
cat  identified_elements.txt  repeatmodeler_identities.fasta  > ModelerID.lib
"""

get_final_lib_shell = """#! /bin/bash
DIR_PE=%s

source activate maker_p

$DIR_PE/ProtExcluder.pl -f 50 ModelerUnknown.lib_blast_results.txt ModelerUnknown.lib
$DIR_PE/ProtExcluder.pl -f 50 MITE.lib_blast_results.txt MITE.lib
$DIR_PE/ProtExcluder.pl -f 50 ModelerID.lib_blast_results.txt ModelerID.lib

sed 's/(//g;s/)//g' allLTR.lib > allLTR.lib.re
sed 's/_(/_/g;s/)_/_/g' allLTR.lib_blast_results.txt > allLTR.lib_re_blast_results.txt

perl $DIR_PE/ProtExcluder.pl -f 50 allLTR.lib_re_blast_results.txt allLTR.lib.re
cat MITE.libnoProtFinal allLTR.lib.renoProtFinal ModelerID.libnoProtFinal > KnownRepeats.lib
cat KnownRepeats.lib ModelerUnknown.libnoProtFinal > allRepeats.lib
"""


jf_file = '/lustre/home/xuyuxing/Database/Gel/other/Aof/sra_data/ERR2040729.jf'

from lib.xuyuxing.base.base_function import cmd_open
from lib.common.util import printer_list
from lib.common.os import cmd_run


def jf_file_parse(jf_file):
    for i in cmd_open("jellyfish dump -c %s" % jf_file):
        print(i)
        kmer, num = i.split()
        num = int(num)
        yield kmer, num


def get_jf_file(input_file_list, jf_file, gzip_flag=False, sra_flag=False):

    if gzip_flag:
        input_file_string = printer_list(input_file_list, sep=" ", head="<(zcat ") + ")"
    else:
        input_file_string = printer_list(input_file_list, sep=" ", head="<(cat ") + ")"
    
    if sra_flag:
        input_file_string = printer_list(input_file_list, sep=" ", head="<(fastq-dump -Z ") + ")"


    cmd_string = "jellyfish count -m 17 -s 100M -t 20 -C -o " + jf_file + " " + input_file_string
    
    cmd_run(cmd_string)

jf_file = '/lustre/home/xuyuxing/Database/Gel/other/Aof/sra_data/ERR2040729.jf'
num = 0
for i in jf_file_parse(jf_file):
    num += 1

def compare_two_jf():
    pass


import pysam
from lib.common.genome.seq_base import read_fastq_big

def get_top_M_score(read_aln_list):
    top_M = 0
    
    for aln in read_aln_list:
        cigar_list = aln.cigar
        TM = sum([i[1] for i in cigar_list if i[0] == 0])
        DM = sum([i[1] for i in cigar_list if not (i[0] == 0 or i[0] == 3)])

        M = TM - DM

        if M > top_M:
            top_M = M
    
    return top_M


def get_best_alignment_match_number(read_aln_list):
    valid_aln_list = [alignedsegment for alignedsegment in read_aln_list if not (alignedsegment.is_secondary or alignedsegment.is_supplementary)]

    # get 1-end
    read1_aln_list = [alignedsegment for alignedsegment in valid_aln_list if alignedsegment.is_read1]

    # get 2-end
    read2_aln_list = [alignedsegment for alignedsegment in valid_aln_list if alignedsegment.is_read2]

    read1_top_M = get_top_M_score(read1_aln_list)
    read2_top_M = get_top_M_score(read2_aln_list)

    M_score = read1_top_M + read2_top_M

    return M_score

def get_tag(read_aln_list1, read_aln_list2, tag1, tag2):
    M1 = get_best_alignment_match_number(read_aln_list1)
    M2 = get_best_alignment_match_number(read_aln_list2)

    if M1 > M2:
        return tag1, M1, M2
    elif M2 > M1:
        return tag2, M1, M2
    else:
        return "X", M1, M2

def extract_aln_by_reads(reads_name, name_indexed):
    output_list = []
    try:
        name_indexed.find(reads_name)
    except KeyError:
        pass
    else:
        iterator = name_indexed.find(reads_name)
        output_list = []
        for x in iterator:
            output_list.append(x)
    
    return output_list

def index_bam_file(bam_file):
    bamfile = pysam.AlignmentFile(bam_file, 'rb')
    name_indexed = pysam.IndexedReads(bamfile)
    name_indexed.build()
    header = bamfile.header.copy()
    return name_indexed, header

def get_read_name_list_from_fastq(fastq_file_name, output_file_name):
   
    with open(output_file_name, 'w') as f:
        for record in read_fastq_big(fastq_file_name,gzip_flag=True):
            f.write(record[0].seqname_short() + "\n")

    return output_file_name

def MappedReadsComparing_main(args):
    reads_id_file= get_read_name_list_from_fastq(args.fastq_file, args.reads_id_file)


    # indexed the bam file
    name_indexed_1, header1 = index_bam_file(args.mapped_bam_file_1)
    name_indexed_2, header2 = index_bam_file(args.mapped_bam_file_2)

    # remove the None element
    with open(args.tag_file, 'w') as w:
        with open(args.reads_id_file, 'r') as f:
            for each_line in f:
                read_name = each_line.strip()
        
                # extracting the alignmentseqments
                read_aln_list_1= extract_aln_by_reads(read_name, name_indexed_1)
                read_aln_list_2= extract_aln_by_reads(read_name, name_indexed_2)
        
                tag, M1 , M2  = get_tag(read_aln_list_1, read_aln_list_2, args.tag1, args.tag2)

                w.write("%s\t%s\t%d\t%d\n" % (read_name, tag, M1, M2))


import re
import os
from collections import OrderedDict
import pandas as pd
from BCBio import GFF
from Bio.Seq import Seq
from Bio.SeqRecord import SeqRecord
from interlap import InterLap
from lib.common.util import logging_init
from lib.common.os import have_file, mkdir, rmdir, copy_file, cmd_run, multiprocess_running
from lib.common.math.bin_and_window import split_sequence_to_bins
from lib.xuyuxing.math.set_operating import overlap_with_interlap_set
from lib.common.math.interval import merge_intervals
from lib.common.genome.seq_base import read_fasta_by_faidx, reverse_complement, read_fasta, read_fasta_to_sqlite, extract_seq_from_sqlite
from api.xuyuxing.genome.genblasta import genblasta_run, read_genblasta_results
from lib.common.genome.genome_feature2 import fancy_name_parse, add_gfs_into_db, read_gff_file, get_gf_db_meta_dict, gf_info_retrieval, write_gff_file
from api.xuyuxing.genome.genewise import genewise_run, genewise_output_parser, gff_rec_obj_rename, block_dict_2_gff_rec
from api.xuyuxing.genome.repeatmasker import repeatmasker_parser
import lib.common.sqlite_command as sc
from config import wu_blast_path
from api.common.genome.blast import outfmt6_read_big


def get_skip_file(gff_file, repeat_file, output_file):
    gene_dict = read_gff_file(gff_file)['gene']
    repeat_dict = repeatmasker_parser(repeat_file)

    skiped_range = {}
    for gene_id in gene_dict:
        gene = gene_dict[gene_id]

        if gene.chr_id not in skiped_range:
            skiped_range[gene.chr_id] = []

        for mRNA in gene_dict[gene_id].sub_features:
            for cds in mRNA.sub_features:
                if cds.type == 'CDS':
                    skiped_range[gene.chr_id].append(cds.range)

    for repeat_type in repeat_dict:
        for repeat_family_id in repeat_dict[repeat_type]:
            repeat_family = repeat_dict[repeat_type][repeat_family_id]
            for repeat_case in repeat_family.case_list:

                repeat_case.q_name

                if repeat_case.q_name not in skiped_range:
                    skiped_range[repeat_case.q_name] = []

                skiped_range[repeat_case.q_name].append(
                    (repeat_case.q_start, repeat_case.q_end))

    for contig in skiped_range:
        skiped_range[contig] = merge_intervals(skiped_range[contig])

    with open(output_file, 'w') as f:
        for contig in skiped_range:
            for range_tmp in skiped_range[contig]:
                f.write("%s\t%d\t%d\n" % (contig, range_tmp[0], range_tmp[1]))


def read_skip_file(skip_file):
    skip_range_dict = {}

    if skip_file:

        with open(skip_file, 'r') as f:
            for each_line in f:

                each_line = each_line.strip()
                chr_id, start, end = each_line.split()
                start = int(start)
                end = int(end)

                if chr_id not in skip_range_dict:
                    skip_range_dict[chr_id] = []

                skip_range_dict[chr_id].append((min(start, end), max(start, end)))

        for i in skip_range_dict:
            skip_range_dict[i] = InterLap(skip_range_dict[i])

    return skip_range_dict


def init_query_meta_db(query_seq_list, query_fasta_dict, query_sqlite_db, split):
    # remove old db
    # print(query_sqlite_db)
    if os.path.exists(query_sqlite_db):
        cmd_run("rm %s" % query_sqlite_db)

    # build query_seq_table
    work_dir_hash = {}
    dir_name_hash = {}
    for index, start, end in split_sequence_to_bins(len(query_seq_list), split, start=1):
        dir_name = str(start) + "_" + str(end)
        dir_name_hash[dir_name] = []
        query_seq_in_range = query_seq_list[start - 1: end]

        for q_id in query_seq_in_range:
            dir_name_hash[dir_name].append(q_id)
            work_dir_hash[q_id] = dir_name

    sc.init_sql_db(query_sqlite_db, 'query_seq_table', [
                   'id', 'q_id', 'q_len', 'sub_dir'], remove_old_db=True)

    record_list = [(i, query_seq_list[i], len(query_fasta_dict[query_seq_list[i]].faidx), work_dir_hash[query_seq_list[i]])
                   for i in range(len(query_seq_list))]

    sc.sqlite_write(record_list, query_sqlite_db,
                    'query_seq_table', ['id', 'q_id', 'q_len', 'sub_dir'])

    # gba tables
    gba_col = ['id', 'q_id', 'q_start', 'q_end', 's_contig', 's_start', 's_end', 'strand',
               'rank', 'h_name', 'start_e', 'end_e', 'gws_s_base', 'cover', 'skip_cover', 'gws_check']

    table_columns_dict = {}
    for dir_name in dir_name_hash:
        table_columns_dict['work'+dir_name] = gba_col

    sc.init_sql_db_many_table(
        query_sqlite_db, table_columns_dict, remove_old_db=False)


def build_genblasta_dir_tree(query_dir, query_gene_fasta, query_seq_list, split):
    seqdict, seqname_list = read_fasta(query_gene_fasta)
    query_fasta_dict = {i: seqdict[i].seq for i in seqdict}

    for index, start, end in split_sequence_to_bins(len(query_seq_list), split, start=1):
        dir_name = str(start) + "_" + str(end)

        mkdir(query_dir + "/" + dir_name, True)
        query_seq_in_range = query_seq_list[start - 1: end]

        for q_id in query_seq_in_range:
            q_file = query_dir + "/" + dir_name + "/" + q_id + '.fa'

            if os.path.exists(q_file) and os.path.getsize(q_file) != 0:
                continue
            else:
                with open(q_file, 'w') as f:
                    f.write(">%s\n%s\n" % (q_id, query_fasta_dict[q_id]))


def genblasta_results_num(genblasta_results_file):
    cmd_string = "grep \"rank:\" %s | sed \'s/.*rank://g\'" % genblasta_results_file
    flag, output, error = cmd_run(cmd_string, silence=True)

    output_list = [int(i) for i in output.split("\n") if i != '']

    if len(output_list) > 0:
        return max(output_list)
    else:
        return 0


def genblasta_check(gba_file):
    """return: gba_ok, gba_hit_num"""
    if not os.path.exists(gba_file) or os.path.getsize(gba_file) == 0:
        return False, None
    else:
        hit_num = genblasta_results_num(gba_file)
        return True, hit_num


def genblasta_range_not_in_skip_range(gba_file, skip_range_dict, skip_coverage):
    new_range = []

    for q_id, hit_list in read_genblasta_results(gba_file):
        for hit in hit_list:
            query_range = hit.rangeA
            subject_range = hit.rangeB

            skip_flag = False
            if subject_range.chr_id in skip_range_dict:
                cover_length = 0
                total_length = 0

                for hsp in hit.hsp_list:
                    overlap_ratio, overlap_length, overlap_list = overlap_with_interlap_set(
                        hsp.rangeB.range, skip_range_dict[subject_range.chr_id])
                    total_length += hsp.rangeB.len()
                    cover_length += overlap_length

                coverage_ratio = cover_length / total_length

                if coverage_ratio >= skip_coverage:
                    skip_flag = True

            else:
                coverage_ratio = 0.0

            if not skip_flag:
                query_range.get_fancy_name()
                subject_range.get_fancy_name()
                new_range.append(
                    (query_range.fancy_name, subject_range.fancy_name, hit.cover, coverage_ratio))

    return new_range


def get_dir_name_hash(meta_sqlite_db, reverse=False):
    query_gene_info = sc.sqlite_select(
        meta_sqlite_db, 'query_seq_table', ['q_id', 'sub_dir'])
    dir_name_hash = {}

    if reverse:
        return {q_id: sub_dir for q_id, sub_dir in query_gene_info}
    else:
        for q_id, sub_dir in query_gene_info:
            if sub_dir not in dir_name_hash:
                dir_name_hash[sub_dir] = []
            dir_name_hash[sub_dir].append(q_id)

        return dir_name_hash


def get_gws_s_base(contig, start, end, strand, contig_length, expand_range_ratio=0.1):

    expand_range = int(
        (abs(end - start) + 1) * expand_range_ratio)

    start_e = max(
        1, min(start, end) - expand_range)
    end_e = min(contig_length, max(start, end) + expand_range)

    s_base = start_e
    if strand == "-":
        s_base = end_e

    return s_base, start_e, end_e


def genblasta_dir_results_check(query_dir, target_fasta_file, skip_range_file, num_threads, log_file, skip_coverage, genblasta_hit_num):
    annotated_range_dict = {}

    query_sqlite_db = query_dir + "/genblasta.db"

    dir_name_hash = get_dir_name_hash(query_sqlite_db)

    for sub_dir in dir_name_hash:

        # check gba

        parsed_query_gene_list = list(
            set([i[0] for i in sc.sqlite_select(query_sqlite_db, 'work'+sub_dir, ['q_id'])]))
        unparsed_query_gene_list = list(
            set(dir_name_hash[sub_dir]) - set(parsed_query_gene_list))

        check_args_list = []
        for q_id in unparsed_query_gene_list:
            gba_file = query_dir + "/" + sub_dir + "/" + q_id + ".fa.gba"
            check_args_list.append((gba_file,))

        check_out = multiprocess_running(genblasta_check, check_args_list, num_threads,
                                         log_file=log_file, args_id_list=unparsed_query_gene_list)

        check_ok_list = [i for i in check_out if check_out[i]['output'][0]]

        # compare gff

        compare_args_list = []

        if len(check_ok_list) > 0 and len(annotated_range_dict) == 0:
            skip_range_dict = read_skip_file(skip_range_file)

        # for q_id in check_ok_list:
        #     gba_file = query_dir + "/" + sub_dir + "/" + q_id + ".fa.gba"
        #     compare_args_list.append(
        #         (gba_file, skip_range_dict, skip_coverage))

        # compare_out = multiprocess_running(genblasta_range_not_in_skip_range, compare_args_list,
        #                                    num_threads, log_file=log_file, args_id_list=check_ok_list)

        compare_out = {}
        for q_id in check_ok_list:
            gba_file = query_dir + "/" + sub_dir + "/" + q_id + ".fa.gba"
            compare_out[q_id] = {}
            compare_out[q_id]['output'] = genblasta_range_not_in_skip_range(
                gba_file, skip_range_dict, skip_coverage)

        # write sqlite

        record_id_now = sc.sql_table_row_num(query_sqlite_db, 'work'+sub_dir)
        target_fasta_dict = read_fasta_by_faidx(target_fasta_file)

        waitting_for_load = []
        for q_id in check_ok_list:

            if len(compare_out[q_id]['output']) == 0:
                waitting_for_load.append(
                    (record_id_now, q_id, 0, 0, "None", 0, 0, ".", 0, "None", 0, 0, 0, 0.0, 0.0, True))
                record_id_now += 1
            else:
                h_num = 0
                for q_range, h_range, cover, skip_cover in compare_out[q_id]['output']:
                    s_contig, s_start, s_end, s_strand = fancy_name_parse(
                        h_range)
                    q_contig, q_start, q_end, q_strand = fancy_name_parse(
                        q_range)

                    h_name = q_id + "_" + str(h_num)

                    s_base, start_e, end_e = get_gws_s_base(
                        s_contig, s_start, s_end, s_strand, len(target_fasta_dict[s_contig].faidx), 0)

                    waitting_for_load.append(
                        (record_id_now, q_id, q_start, q_end, s_contig, s_start, s_end, s_strand, h_num, h_name, start_e, end_e, s_base, cover, skip_cover, False))
                    h_num += 1
                    record_id_now += 1

                    if h_num >= genblasta_hit_num:
                        break

            if len(waitting_for_load) > 10000:
                sc.sqlite_write(waitting_for_load, query_sqlite_db, 'work'+sub_dir, [
                                'id', 'q_id', 'q_start', 'q_end', 's_contig', 's_start', 's_end', 'strand', 'rank', 'h_name', 'start_e', 'end_e', 'gws_s_base', 'cover', 'skip_cover', 'gws_check'])
                waitting_for_load = []

        if not len(waitting_for_load) == 0:
            sc.sqlite_write(waitting_for_load, query_sqlite_db, 'work'+sub_dir, [
                            'id', 'q_id', 'q_start', 'q_end', 's_contig', 's_start', 's_end', 'strand', 'rank', 'h_name', 'start_e', 'end_e', 'gws_s_base', 'cover', 'skip_cover', 'gws_check'])

        del waitting_for_load


def seq_stats(meta_sqlite_db, check_key=None, return_key='q_id'):
    table_list = list(sc.check_sql_table(meta_sqlite_db))
    table_list.remove('query_seq_table')

    big_output_list = []

    for table_name in table_list:

        if check_key is None:
            output_list = sc.sqlite_select(
                meta_sqlite_db, table_name, column_list=[return_key])
        else:
            output_list = sc.sqlite_select(meta_sqlite_db, table_name, column_list=[
                return_key], key_name=check_key, value_tuple=(True,))

        output_list = list(set([i[0] for i in output_list]))

        big_output_list.extend(output_list)

    return list(set(big_output_list))


def read_gws_file(gws_file, query_length, strand, gws_base, h_name, q_name, s_name, output_gws_db, output_pep_file, output_cds_file):
    """
    gws_dir = '/lustre/home/xuyuxing/Work/Gel/Gene_Loss/plant/all_map/test/Cau/running_dir/genblasta_genewise/13333/1_100/T13333N0C0000G00034/0'
    output_gws_db = '/lustre/home/xuyuxing/Work/Gel/Gene_Loss/plant/all_map/test/Cau/running_dir/genblasta_genewise/13333/genewise.db'
    output_pep_file = '/lustre/home/xuyuxing/Work/Gel/Gene_Loss/plant/all_map/test/Cau/running_dir/genblasta_genewise/13333/genewise.pep.fasta'
    output_cds_file = '/lustre/home/xuyuxing/Work/Gel/Gene_Loss/plant/all_map/test/Cau/running_dir/genblasta_genewise/13333/genewise.cds.fasta'
    """

    if not os.path.exists(gws_file) or os.path.getsize(gws_file) == 0:
        return False, None, None, None

    try:

        gff_dict = {}
        pro_dict = {}
        cds_dict = {}

        block_dict, gff_output, block_pro_seq, block_cds_seq = genewise_output_parser(
            gws_file)

        # stat
        # [re.sub(' ', '', a[0][i]['block'][0]) for i in a[0]]
        block_dict

        aln_len = sum([len(re.sub(' ', '', block_dict[i]['block'][0]))
                       for i in block_dict])

        pst_len = sum([len(re.sub(' ', '', block_dict[i]['block'][1]))
                       for i in block_dict])

        idt_len = sum([len(re.sub('[ +]', '', block_dict[i]['block'][1]))
                       for i in block_dict])

        contig_rec = block_dict_2_gff_rec(block_dict, gff_output, query_length)

        if strand == "+":
            strand = 1
        else:
            strand = -1

        renamed_contig_rec = gff_rec_obj_rename(
            contig_rec, hit_name=h_name, q_name=q_name, s_name=s_name, s_base=gws_base, strand=strand)

        rec = SeqRecord(Seq(""), s_name)
        gff_dict[s_name] = rec

        gff_dict[s_name].features.append(
            renamed_contig_rec.features[0])

        pro_dict[h_name] = block_pro_seq
        cds_dict[h_name] = block_cds_seq

        gff_file = gws_file + ".gff3"

        rec_obj_list = [gff_dict[i] for i in gff_dict]
        with open(gff_file, "w") as f:
            GFF.write(rec_obj_list, f)

        gff_dict = read_gff_file(gff_file)

        for i in gff_dict:
            for j in gff_dict[i]:
                gf = gff_dict[i][j]

        gf.sub_features[0].qualifiers['aln_len'] = aln_len
        gf.sub_features[0].qualifiers['pst_len'] = pst_len
        gf.sub_features[0].qualifiers['idt_len'] = idt_len

        os.remove(gff_file)

        return True, gf, cds_dict, pro_dict

    except:

        return False, None, None, None


def get_q_len_from_db(query_sqlite_db):
    output_dict = {}
    for q_id, q_len in sc.sqlite_select(query_sqlite_db, 'query_seq_table', ['q_id', 'q_len']):
        output_dict[q_id] = q_len
    return output_dict


def genewise_dir_results_check(query_dir, query_sqlite_db, sub_dir, q_len_dict, output_gws_db, output_pep_file, output_cds_file, num_threads, log_file, clean_success=True):

    args_list = []
    id_list = []

    for (id_tmp, q_id, q_start, q_end, s_contig, start, end, strand, rank, h_name, start_e, end_e, gws_s_base, cover, skip_cover, gws_check) in sc.sqlite_select(query_sqlite_db, 'work'+sub_dir, None, 'gws_check', (0,)):
        output_gws_file = query_dir + "/" + sub_dir + \
            "/" + q_id + "/" + str(rank) + "/output.gws"
        id_list.append(id_tmp)
        args_list.append((output_gws_file, q_len_dict[q_id], strand, gws_s_base,
                          h_name, q_id, s_contig, output_gws_db, output_pep_file, output_cds_file))

    check_out = multiprocess_running(
        read_gws_file, args_list, num_threads, log_file=log_file, args_id_list=id_list)

    gf_big_list = []
    for id_tmp in check_out:
        if check_out[id_tmp]['output'][0]:
            flag, gf, cds_dict, pro_dict = check_out[id_tmp]['output']
            gf_big_list.append(gf)
            with open(output_cds_file, 'a') as f:
                for i in cds_dict:
                    f.write(">%s\n%s\n" % (i, cds_dict[i]))

            with open(output_pep_file, 'a') as f:
                for i in pro_dict:
                    f.write(">%s\n%s\n" % (i, pro_dict[i]))

    add_gfs_into_db(gf_big_list, output_gws_db, max_table_row=10000)

    gws_check_out_list = [(id_tmp, check_out[id_tmp]['output'][0])
                          for id_tmp in check_out if check_out[id_tmp]['output'][0]]

    sc.sqlite_update(query_sqlite_db, 'work'+sub_dir,
                     'id', 'gws_check', gws_check_out_list)

    # clean dir
    if clean_success:
        for (id_tmp, q_id, q_start, q_end, s_contig, start, end, strand, rank, h_name, start_e, end_e, gws_s_base, cover, skip_cover, gws_check) in sc.sqlite_select(query_sqlite_db, 'work'+sub_dir, None, 'gws_check', (1,)):
            query_prefix_path = query_dir + "/" + sub_dir + "/" + q_id

            fa_file = query_prefix_path + ".fa"
            fai_file = query_prefix_path + ".fai"
            gba_file = query_prefix_path + ".fa.gba"
            gba_report = query_prefix_path + ".fa.gba.report"
            wublast_file = query_prefix_path + ".fa.wublast"
            gws_dir = query_prefix_path

            rmdir(fa_file)
            rmdir(fai_file)
            rmdir(gba_file)
            rmdir(gba_report)
            rmdir(wublast_file)
            rmdir(gws_dir)


def prepare_genewise_running(query_sqlite_db, query_dir, sub_dir, query_seq_dict, target_genome_dict):

    genewise_args_list = []

    for (id_tmp, q_id, q_start, q_end, s_contig, start, end, strand, rank, h_name, start_e, end_e, gws_s_base, cover, skip_cover, gws_check) in sc.sqlite_select(query_sqlite_db, 'work'+sub_dir, None, 'gws_check', (0,)):
        query_gws_dir = query_dir + "/" + sub_dir + "/" + q_id + "/"
        mkdir(query_gws_dir, True)
        query_rank_gws_dir = query_gws_dir + "/" + str(rank) + "/"
        mkdir(query_rank_gws_dir, True)

        seq_str = target_genome_dict[s_contig][start_e - 1: end_e].upper()

        if strand == "-":
            seq_str = reverse_complement(seq_str)

        if not (os.path.exists(query_rank_gws_dir + "/subject.fa") and os.path.getsize(
                query_rank_gws_dir + "/subject.fa") != 0):
            with open(query_rank_gws_dir + "/subject.fa", 'w') as f:
                f.write(">subject\n%s\n" % seq_str)
        if not (os.path.exists(query_rank_gws_dir + "/query.fa") and os.path.getsize(
                query_rank_gws_dir + "/query.fa") != 0):
            with open(query_rank_gws_dir + "/query.fa", 'w') as f:
                f.write(">query\n%s\n" % query_seq_dict[q_id])

        genewise_args_list.append((query_rank_gws_dir + "/query.fa", query_rank_gws_dir +
                                   "/subject.fa", query_rank_gws_dir + "/output.gws", '+', True, q_start, q_end))

    return genewise_args_list


def build_WPGmapper(args):

    # init project
    # print arguments to log file
    args.work_dir = os.path.abspath(args.work_dir)
    mkdir(args.work_dir, True)

    args.log_file = args.work_dir + "/log"

    logger = logging_init("WideProteinToGenomeMaper", args.log_file)

    args_info_string = "Argument list:\n"
    attrs = vars(args)

    for item in attrs.items():
        args_info_string = args_info_string + ("%s: %s\n" % item)

    logger.info(args_info_string)

    # read query info table
    query_db = pd.read_excel(args.reference_genome_table)
    query_file_dict = {}
    for index in query_db.index:
        query_id = str(query_db.loc[index]['sp_id'])
        query_pt_file = str(query_db.loc[index]['pt_file'])

        if not pd.isna(query_id) and not pd.isna(query_pt_file):
            query_file_dict[query_id] = query_pt_file

    diamond_dir = args.work_dir + "/diamond_dir"
    huge_ref_pt_file = diamond_dir + "/huge_ref_pt.fasta"
    diamond_db = huge_ref_pt_file + ".dmnd"
    if args.force_redo or not os.path.exists(diamond_dir):
        mkdir(diamond_dir, True)

        for query_id in query_file_dict:
            pt_file = query_file_dict[query_id]
            cmd_string = "cat %s >> %s" % (pt_file, huge_ref_pt_file)
            cmd_run(cmd_string, silence=True)

        cmd_string = "diamond makedb --in %s --db %s" % (
            huge_ref_pt_file, huge_ref_pt_file)
        cmd_run(cmd_string, silence=True)

    printer_string = ""
    for i in query_file_dict:
        printer_string += "ID: %s\tfile: %s\n" % (i, query_file_dict[i])
    logger.info("There are %d query genome files: \n" %
                len(query_file_dict) + printer_string)

    # copy target_genome_fasta to work_dir
    logger.info("copy target genome to work dir\n")
    args.target_genome_fasta = copy_file(
        args.target_genome_fasta, args.work_dir, True)

    # copy target_genome_gff to work_dir
    logger.info("copy skip_range_file to work dir\n")
    if args.skip_range_file:
        args.skip_range_file = copy_file(
            args.skip_range_file, args.work_dir, True)

    # build dir tree
    logger.info("Build work dir tree\n")

    run_dir = args.work_dir + "/WPGmapper"

    mkdir(run_dir, True)

    for i in query_file_dict:
        mkdir(run_dir + "/" + i, True)
        query_file_dict[i] = copy_file(
            query_file_dict[i], run_dir + "/" + i, True)

    # genblasta
    logger.info("Genblasta: ")
    # build genblasta dir tree
    logger.info("build genblasta dir tree")

    for query_speci_id in query_file_dict:
        logger.info("work on %s" % query_speci_id)

        query_fasta_dict = read_fasta_by_faidx(query_file_dict[query_speci_id])
        query_seq_list = list(query_fasta_dict.keys())

        query_dir = "%s/%s" % (run_dir, query_speci_id)

        query_sqlite_db = "%s/%s/genblasta.db" % (run_dir, query_speci_id)

        # create sqlite db
        if args.force_redo or not os.path.exists(query_sqlite_db):
            init_query_meta_db(
                query_seq_list, query_fasta_dict, query_sqlite_db, args.split_seq_num)

        # build dir tree split sequence to file
        dir_file_list = os.listdir(query_dir)
        ok_flag = True
        for i in [str(i[1])+"_"+str(i[2]) for i in split_sequence_to_bins(len(query_seq_list), args.split_seq_num, start=1)]:
            if i not in dir_file_list:
                ok_flag = False

        if args.force_redo or ok_flag is False:
            build_genblasta_dir_tree(
                query_dir, query_file_dict[query_speci_id], query_seq_list, args.split_seq_num)

    # running genblasta
    logger.info("running genblasta\n")
    cmd_string = "%s/xdformat -n %s" % (wu_blast_path,
                                        args.target_genome_fasta)
    tmp_out = cmd_run(cmd_string, silence=True)

    for query_speci_id in query_file_dict:
        logger.info("\tparse %s" % query_speci_id)

        query_dir = "%s/%s" % (run_dir, query_speci_id)
        meta_sqlite_db = query_dir+"/genblasta.db"

        query_fasta_dict = read_fasta_by_faidx(query_file_dict[query_speci_id])
        query_seq_list = list(query_fasta_dict.keys())

        genblasta_dir_results_check(query_dir, args.target_genome_fasta,
                                    args.skip_range_file, args.num_threads, None, args.annotated_coverage, args.genblasta_hit_num)

        gba_passed_id_list = seq_stats(meta_sqlite_db)

        un_gba_query_gene_list = list(
            set(query_seq_list) - set(gba_passed_id_list))

        logger.info("\tThere are %s genes: %s already have good hits, %s need run now" % (
            len(query_seq_list), len(gba_passed_id_list), len(un_gba_query_gene_list)))

        dir_name_hash = get_dir_name_hash(meta_sqlite_db, True)

        args_list = []
        for q_id in un_gba_query_gene_list:
            q_file = query_dir + "/" + dir_name_hash[q_id] + "/" + q_id + ".fa"
            gba_file = q_file + ".gba"
            args_list.append((q_file, args.target_genome_fasta,
                              gba_file, args.gene_coverage, args.genblasta_hit_num))

        genblasta_out = multiprocess_running(
            genblasta_run, args_list, args.num_threads, log_file=args.log_file, timeout=1800)

        genblasta_dir_results_check(query_dir, args.target_genome_fasta,
                                    args.skip_range_file, args.num_threads, None, args.annotated_coverage, args.genblasta_hit_num)

        gba_passed_id_list = seq_stats(meta_sqlite_db)
        un_gba_query_gene_list = list(
            set(query_seq_list) - set(gba_passed_id_list))

        logger.info("After genblasta: %s genes have good hits, %s failed" % (
            len(gba_passed_id_list), len(un_gba_query_gene_list)))

    logger.info("running genewise files")

    target_genome_dict = read_fasta_by_faidx(args.target_genome_fasta)
    target_genome_dict = {
        i: str(target_genome_dict[i].seq) for i in target_genome_dict}

    for query_speci_id in query_file_dict:
        query_dir = "%s/%s" % (run_dir, query_speci_id)
        logger.info("\tparse %s" % query_speci_id)
        query_seq_dict = read_fasta_by_faidx(query_file_dict[query_speci_id])
        query_seq_dict = {i: str(query_seq_dict[i].seq)
                          for i in query_seq_dict}

        query_sqlite_db = query_dir + "/genblasta.db"
        dir_name_hash = get_dir_name_hash(query_sqlite_db)
        dir_name_hash_reverse = get_dir_name_hash(query_sqlite_db, True)
        q_len_dict = get_q_len_from_db(query_sqlite_db)

        output_gws_db = query_dir + "/genewise.db"
        output_pep_file = query_dir + "/genewise.pep.fa"
        output_cds_file = query_dir + "/genewise.cds.fa"

        for sub_dir in dir_name_hash:
            logger.info("\tparse sub_dir: %s" % sub_dir)

            # parse old genewise output
            genewise_dir_results_check(query_dir, query_sqlite_db, sub_dir, q_len_dict, output_gws_db,
                                       output_pep_file, output_cds_file, args.num_threads, None, clean_success=True)

            # prepare genewise input file
            genewise_args_list = prepare_genewise_running(
                query_sqlite_db, query_dir, sub_dir, query_seq_dict, target_genome_dict)

            # running genewise
            multiprocess_running(
                genewise_run, genewise_args_list, args.num_threads, log_file=args.log_file, timeout=1800)

            # reparse genewise output
            genewise_dir_results_check(query_dir, query_sqlite_db, sub_dir, q_len_dict, output_gws_db,
                                       output_pep_file, output_cds_file, args.num_threads, None, clean_success=True)

        # make diamond for pep seq
        logger.info("\tmake diamond%s" % sub_dir)

        diamond_out = query_dir + "/genewise.pep.bls"
        if os.path.exists(diamond_out):
            os.remove(diamond_out)
        cmd_string = "diamond blastp -q %s -k 100 -e 1e-5 -d %s -o %s -f 6 -p %d" % (
            output_pep_file, diamond_db, diamond_out, args.num_threads)
        cmd_run(cmd_string, silence=True)

        # write fasta into sqlite3
        output_pep_db = query_dir + "/genewise.pep.db"
        read_fasta_to_sqlite(output_pep_file, output_pep_db, False)

        output_cds_db = query_dir + "/genewise.cds.db"
        read_fasta_to_sqlite(output_cds_file, output_cds_db, False)


def parse_diamond_out(diamond_file):

    ev_diamond_info = {}
    for query_record in outfmt6_read_big(diamond_file):
        q_id = query_record.qID
        ev_diamond_info[q_id] = {}

        q_parent_id = re.match('^(.*)_\d+$',q_id).groups()[0]

        for hit_record in query_record.hit:
            hit_id = hit_record.Hit_id
            if hit_id == q_parent_id:
                ev_diamond_info[q_id]["hit_rank"] = hit_record.Hit_num
                top_hsp = hit_record.hsp[0]
                ev_diamond_info[q_id]["identity"] = top_hsp.Hsp_identical_ratio
                ev_diamond_info[q_id]["bit_score"] = top_hsp.Hsp_bit_score
                ev_diamond_info[q_id]["evalue"] = top_hsp.Hsp_evalue

    return ev_diamond_info


def extract_one_evidences(id_tmp, gf_name, type_tmp, contig_name, start, end, strand, daughter, qualifiers, gws_db, OG_support="", taxon_id="", table_name=""):
    gf = gf_info_retrieval((id_tmp, gf_name, type_tmp, contig_name,
                            start, end, strand, daughter, qualifiers), 0, 'A', gws_db)

    mRNA_gf = gf.sub_features[0]
    qualifiers = mRNA_gf.qualifiers

    # query coverage
    coverage = min((int(qualifiers['Target_End'][0]) - int(
        qualifiers['Target_Start'][0]) + 1) / int(qualifiers['Target_Length'][0]), 1.00)

    # aln aa
    aln_aa_len = int(
        qualifiers['Target_End'][0]) - int(qualifiers['Target_Start'][0]) + 1

    # identity
    identity = int(qualifiers['idt_len']) / \
        int(qualifiers['aln_len'])

    q_id = re.match('^(.*)_\d+$',gf_name).groups()[0]

    gf.evidence_indicator = {
        "taxon_id": taxon_id,
        "query_id": q_id,
        "query_coverage": coverage,
        "aln_aa_len": aln_aa_len,
        "identity": identity,
        "query_length": int(qualifiers['Target_Length'][0]),
        "score": max(0.1, float(gf.sub_features[0].qualifiers['score'][0])),
        "OG_support": OG_support
    }

    gf.score_dict = {
        "score": max(0.1, float(gf.sub_features[0].qualifiers['score'][0]))
    }

    gf.db_path = "%s.%s.%s" % (taxon_id, table_name, id_tmp)

    return gf


def extract_all_evidences(ref_WPGmapper_dict, evidence_id_list=None, seq_support_dict=None, num_thread=20, log_file=None):
    module_log = logging_init("extract_all_evidences", log_file)

    # get all useable evidence

    module_log.info('\tBegin: extract parent diamond blast results')
    diamond_args_list = []

    for taxon_id in ref_WPGmapper_dict:
        diamond_file = ref_WPGmapper_dict[taxon_id]['diamond']
        diamond_args_list.append((diamond_file, ))

    diamond_output = multiprocess_running(
        parse_diamond_out, diamond_args_list, num_thread, silence=True, log_file=log_file)

    diamond_seq_info = {}
    num = 0
    for i in diamond_output:
        for j in diamond_output[i]['output']:
            num += 1
            if len(diamond_output[i]['output'][j]) > 0:
                diamond_seq_info[j] = diamond_output[i]['output'][j]

    module_log.info('\tEnd: extract parent diamond blast results')

    module_log.info('\tBegin: extract evidence gf from database')

    if not evidence_id_list is None:
        evidence_id_dict = {i: 0 for i in evidence_id_list}

    ev_args_list = []
    ev_args_id_list = []

    for taxon_id in ref_WPGmapper_dict:
        # print(taxon_id)

        gws_db = ref_WPGmapper_dict[taxon_id]['genewise']
        meta_dict = get_gf_db_meta_dict(gws_db)

        # read mRNA info

        for table_name in meta_dict['gene'][0]:
            # print(table_name)
            sql_cmd_string = 'SELECT * FROM %s' % table_name

            data_list = sc.sqlite_execute(sql_cmd_string, gws_db)
            for gf_db_info_tuple in data_list:
                id_tmp, gf_name, type_tmp, contig_name, start, end, strand, daughter, qualifiers = gf_db_info_tuple

                if not evidence_id_list is None:
                    if gf_name not in evidence_id_dict:
                        continue

                # if gf_name not in diamond_seq_info:
                #     continue

                q_id = gf_name.split("_")[0]

                OG_support = (0, 0)
                if not seq_support_dict is None:
                    if q_id in seq_support_dict:
                        OG_support = seq_support_dict[q_id]

                ev_args_list.append((id_tmp, gf_name, type_tmp, contig_name, start, end,
                                     strand, daughter, qualifiers, gws_db, OG_support, taxon_id, table_name))
                ev_args_id_list.append(gf_name)

    ev_output = multiprocess_running(
        extract_one_evidences, ev_args_list, num_thread, silence=True, args_id_list=ev_args_id_list, log_file=log_file)

    evidence_dict_tmp = OrderedDict()

    for i in ev_output:
        gf = ev_output[i]['output']
        gf.parent_blast = {"hit_rank": 0, "identity": 0,
                           "bit_score": 0, "evalue": 100}
        gf.score_dict["bit_score"] = 0.0
        if i in diamond_seq_info:
            gf.parent_blast = diamond_seq_info[i]
            gf.score_dict["bit_score"] = diamond_seq_info[i]["bit_score"]
        evidence_dict_tmp[i] = gf

    if not evidence_id_list is None:
        evidence_dict = OrderedDict()
        for i in evidence_id_list:
            if i in evidence_dict_tmp:
                evidence_dict[i] = evidence_dict_tmp[i]
    else:
        evidence_dict = evidence_dict_tmp

    module_log.info('\tEnd: extract evidence gf from database')

    # get aa cds seq
    module_log.info('\tBegin: extract AA and cds sequence for each gf')
    species_tmp_dict = {}
    for i in evidence_dict:
        taxon_id = evidence_dict[i].evidence_indicator['taxon_id']
        if taxon_id not in species_tmp_dict:
            species_tmp_dict[taxon_id] = []
        species_tmp_dict[taxon_id].append(i)

    gf_aa_dict = {}
    gf_cds_dict = {}
    for taxon_id in species_tmp_dict:
        gf_aa_dict[taxon_id] = extract_seq_from_sqlite(
            species_tmp_dict[taxon_id], ref_WPGmapper_dict[taxon_id]['pep_db'])
        gf_cds_dict[taxon_id] = extract_seq_from_sqlite(
            species_tmp_dict[taxon_id], ref_WPGmapper_dict[taxon_id]['cds_db'])

    for i in evidence_dict:
        gf = evidence_dict[i]
        taxon_id = gf.evidence_indicator['taxon_id']
        gf.model_aa_seq = gf_aa_dict[taxon_id][i]
        gf.model_cds_seq = gf_cds_dict[taxon_id][i]
        evidence_dict[i] = gf

    del module_log.handlers[:]

    return evidence_dict


def load_map_files(ref_taxon_id_list, WPGmapper_dir):

    # load map database
    ref_WPGmapper_dict = {}
    for taxon_id in ref_taxon_id_list:
        gba_db_file = WPGmapper_dir + "/WPGmapper/" + taxon_id + "/genblasta.db"
        gw_db_file = WPGmapper_dir + "/WPGmapper/" + taxon_id + "/genewise.db"
        gw_pep_file = WPGmapper_dir + "/WPGmapper/" + taxon_id + "/genewise.pep.db"
        gw_cds_file = WPGmapper_dir + "/WPGmapper/" + taxon_id + "/genewise.cds.db"
        gw_pep_bls = WPGmapper_dir + "/WPGmapper/" + taxon_id + "/genewise.pep.bls"

        if have_file(gba_db_file) and have_file(gw_db_file) and have_file(gw_pep_file) and have_file(gw_cds_file) and have_file(gw_pep_bls):
            ref_WPGmapper_dict[taxon_id] = {}
            ref_WPGmapper_dict[taxon_id]['genblasta'] = gba_db_file
            ref_WPGmapper_dict[taxon_id]['genewise'] = gw_db_file
            ref_WPGmapper_dict[taxon_id]['pep_db'] = gw_pep_file
            ref_WPGmapper_dict[taxon_id]['cds_db'] = gw_cds_file
            ref_WPGmapper_dict[taxon_id]['diamond'] = gw_pep_bls

    return ref_WPGmapper_dict


def extract_all_evidences_from_one_gws_db(genewise_db):
    ev_args_list = []
    ev_args_id_list = []

    meta_dict = get_gf_db_meta_dict(genewise_db)

    # read mRNA info

    for table_name in meta_dict['gene'][0]:
        # print(table_name)
        sql_cmd_string = 'SELECT * FROM %s' % table_name

        data_list = sc.sqlite_execute(sql_cmd_string, genewise_db)
        for gf_db_info_tuple in data_list:
            id_tmp, gf_name, type_tmp, contig_name, start, end, strand, daughter, qualifiers = gf_db_info_tuple

            q_id = gf_name.split("_")[0]

            ev_args_list.append((id_tmp, gf_name, type_tmp, contig_name, start, end,
                                    strand, daughter, qualifiers, genewise_db))
            ev_args_id_list.append(gf_name)

    ev_output = multiprocess_running(
        extract_one_evidences, ev_args_list, 56, silence=False, args_id_list=ev_args_id_list)

    output_dict = {}
    for i in ev_output:
        output_dict[i] = ev_output[i]['output']

    return output_dict

def get_rep_gff(genewise_db, gff_file):
    gf_dict = extract_all_evidences_from_one_gws_db(genewise_db)

    query_gf_dict = {}
    for gf_id in gf_dict:
        gf = gf_dict[gf_id]
        q_id = gf.evidence_indicator['query_id']
        if not q_id in query_gf_dict:
            query_gf_dict[q_id] = []
        query_gf_dict[q_id].append(gf)

    best_gf_list = []
    for q_id in query_gf_dict:
        gf_list = query_gf_dict[q_id]
        best_gf = max(gf_list,key=lambda x:x.evidence_indicator['score'])
        best_gf.qualifiers['query'] = q_id
        best_gf.qualifiers['q_cov'] = best_gf.evidence_indicator['query_coverage'] * 100
        best_gf.qualifiers['q_idt'] = best_gf.evidence_indicator['identity'] * 100

        if 'Note' in best_gf.sub_features[0].qualifiers:
            best_gf.qualifiers['Note'] = 'pseudogene'

        best_gf_list.append(best_gf)

    write_gff_file(best_gf_list, rep_gf_gff)


if __name__ == "__main__":

    class abc():
        pass

    args = abc()

    args.num_threads = 56
    args.gene_coverage = 0.2
    args.annotated_coverage = 0.8
    args.genblasta_hit_num = 100
    args.split_seq_num = 100
    args.force_redo = False

    args.work_dir = "/lustre/home/xuyuxing/Work/Gel/Gene_Loss/plant/all_map/test/Cau"
    args.skip_range_file = '/lustre/home/xuyuxing/Work/Gel/Gene_Loss/plant/all_map/test/T267555N0.genome.gff3'
    args.reference_genome_table = '/lustre/home/xuyuxing/Work/Gel/Gene_Loss/plant/all_map/test/pt_file_table_tmp.xlsx'
    args.target_genome_fasta = '/lustre/home/xuyuxing/Work/Gel/Gene_Loss/plant/all_map/test/T267555N0.genome.fasta'

    build_WPGmapper(args)

    ref_WPGmapper_dict = load_map_files(
        ['T4081N0'], '/lustre/home/xuyuxing/Work/Gel/WPGmapper/Cau_WPGmapper')
    extract_all_evidences(ref_WPGmapper_dict, evidence_id_list=None,
                          seq_support_dict=None, num_thread=20, log_file=None)

    # extract genewise db to gff file
    genewise_db = '/lustre/home/xuyuxing/Database/Gel/genome/annotation/v3.0/yuan_pt_vs_xu/WPG_working/WPGmapper/yuan/genewise.db'
    genewise_pep = '/lustre/home/xuyuxing/Database/Gel/genome/annotation/v3.0/yuan_pt_vs_xu/WPG_working/WPGmapper/yuan/genewise.pep.db'
    genewise_cds = '/lustre/home/xuyuxing/Database/Gel/genome/annotation/v3.0/yuan_pt_vs_xu/WPG_working/WPGmapper/yuan/genewise.cds.db'
    rep_gf_gff = '/lustre/home/xuyuxing/Database/Gel/genome/annotation/v3.0/yuan_pt_vs_xu/WPG_working/WPGmapper/yuan/genewise.best.gff'

    get_rep_gff(genewise_db, rep_gf_gff)





import sys

sys.path.append("/lustre/home/xuyuxing/python_project/Genome_work_tools/")
import re

raw_tab_dir = "/lustre/home/xuyuxing/Work/Other/Pseudouracil/raw_tab/"

output_col_1 = ["Gene_id", "site"]
output_col_2 = ["S02_S01", "S06_S05", "S10_S26", "S25_S09", "S04_S03", "S08_S07", "S12_S11", "S28_S27"]
output_col_3 = ["S02", "S01", "S06", "S05", "S10", "S26", "S25", "S09", "S04", "S03", "S08", "S07", "S12", "S11", "S28",
                "S27"]

gene_dir = {
    "Chr2_18S": ('Chr2', 3706, 5781, '+'),
    "Chr2_5.5S": ('Chr2', 5782, 6133, '+'),
    "Chr2_25S": ('Chr2', 6134, 9608, '+'),
    "Chr3_18S": ('Chr3', 14197677, 14199752, '+'),
    "Chr3_5.5S": ('Chr3', 14199753, 14200104, '+'),
    "Chr3_25S": ('Chr3', 14200105, 14203579, '+')
}

coverage_dir = {}
for i in output_col_3:
    file_name = raw_tab_dir + i + ".tab"
    mapped_read_num = 0
    with open(file_name, 'r') as f:
        for each_line in f:
            each_line = re.sub('\n', '', each_line)
            line_data = re.split('\t', each_line)
            line_chr = line_data[0]
            line_site = int(line_data[1])
            line_p_e = int(line_data[2])
            line_p_p = int(line_data[3])
            line_n_e = int(line_data[4])
            line_n_p = int(line_data[5])
            mapped_read_num = mapped_read_num + line_p_e + line_n_e
            for gene_id in gene_dir:
                gene_info = gene_dir[gene_id]
                gene_chr = gene_info[0]
                gene_s = gene_info[1]
                gene_e = gene_info[2]
                gene_strand = gene_info[3]
                if line_chr == gene_chr:
                    if line_site in list(range(gene_s, gene_e)):
                        if gene_strand == "+":
                            coverage = line_p_e
                        else:
                            coverage = line_n_e
                        if not gene_id in coverage_dir:
                            coverage_dir[gene_id] = {}
                        if not i in coverage_dir[gene_id]:
                            coverage_dir[gene_id][i] = {}
                        coverage_dir[gene_id][i][line_site] = coverage
    print(i, mapped_read_num)
    for gene_id in coverage_dir:
        for site in coverage_dir[gene_id][i]:
            coverage_dir[gene_id][i][site] = coverage_dir[gene_id][i][site] * 1000000 / mapped_read_num

tran_dir = '/lustre/home/xuyuxing/Work/Other/Pseudouracil/all_tran/'

peak_dir = {}
for i in output_col_2:
    file_name = tran_dir + i + ".all_transcript.tab"
    with open(file_name, 'r') as f:
        for each_line in f:
            each_line = re.sub('\n', '', each_line)
            line_data = re.split('\t', each_line)
            line_chr = line_data[0]
            line_gene_id = line_data[1]
            line_site = int(line_data[2])
            line_peak_p = float(line_data[3])
            line_peak_n = float(line_data[4])
            for gene_id in gene_dir:
                gene_info = gene_dir[gene_id]
                gene_chr = gene_info[0]
                gene_s = gene_info[1]
                gene_e = gene_info[2]
                gene_strand = gene_info[3]
                if line_chr == gene_chr:
                    if line_site in list(range(gene_s, gene_e)):
                        peak = line_peak_p
                        if not gene_id in peak_dir:
                            peak_dir[gene_id] = {}
                        if not i in peak_dir[gene_id]:
                            peak_dir[gene_id][i] = {}
                        peak_dir[gene_id][i][line_site] = peak
    print(i)

with open("output.txt", 'w') as f:
    for gene_id in gene_dir:
        gene_chr, gene_s, gene_e, gene_strand = gene_dir[gene_id]
        for i in range(gene_s, gene_e + 1):
            printer = gene_id + "\t" + str(i) + "\t"
            for j in output_col_2:
                if i in peak_dir[gene_id][j]:
                    printer = printer + str(peak_dir[gene_id][j][i]) + "\t"
                else:
                    printer = printer + "0.0\t"
            for j in output_col_3:
                if i in coverage_dir[gene_id][j]:
                    printer = printer + str(coverage_dir[gene_id][j][i]) + "\t"
                else:
                    printer = printer + "0.0\t"
            printer.rstrip("\t")
            f.write(printer + "\n")



from lib.common.genome.genome_feature2 import read_gff_file
from lib.common.genome.seq_base import read_fasta
import re

repeat_superfamily_dict = {
    'LTR':['LTR/Gypsy', 'LTR/unknown', 'LTR/Copia'],
    'MITE':['MITE/DTM', 'MITE/DTC', 'MITE/DTH', 'MITE/DTA', 'MITE/DTT'],
    'TIR':['DNA/DTM', 'TIR/MuDR_Mutator', 'DNA/DTC', 'TIR/EnSpm_CACTA', 'DNA/DTH', 'TIR/PIF_Harbinger', 'DNA/DTA', 'TIR/hAT', 'DNA/DTT', 'TIR/Tc1_Mariner', ],
    'Helitron':['DNA/Helitron', 'Helitron'],
    'Maverick':['Maverick'],
    'LINE':['LINE/unknown'],
    'Other':['Unknown', 'pararetrovirus'],
}

def get_r_family_name(r_name):
    mlist = re.findall(r'(TE_\d+)', r_name)
    if len(mlist):
        r_name = mlist[0]
    else:
        r_name = None
    return r_name

def read_EDTA_gff(EDTA_gff_file):
    EDTA_gff_dict = read_gff_file(EDTA_gff_file)
    
    repeat_class_dict = {}
    for source in EDTA_gff_dict:
        for r_id in EDTA_gff_dict[source]:
            r = EDTA_gff_dict[source][r_id]
            r_name = r.qualifiers['Name'][0]
            r_family_id = get_r_family_name(r_name)
            if r_family_id:        
                repeat_class = r.qualifiers['Classification'][0]
                repeat_class_dict.setdefault(repeat_class, {}).setdefault(r_family_id,{})
                repeat_class_dict[repeat_class][r_family_id][r_id] = r

    return repeat_class_dict
    
def read_EDTA_lib(EDTA_lib_file):
    
    EDTA_lib_dict = read_fasta(EDTA_lib_file)[0]

    EDTA_lib_seq_dict = {}
    for i in EDTA_lib_dict:
        EDTA_lib_seq_dict[get_r_family_name(i)] = EDTA_lib_dict[i]

    return EDTA_lib_seq_dict



import os
import numpy as np
from lib.common.fileIO import tsv_file_dict_parse_big, read_matrix_file
from lib.common.genome.seq_base import read_fasta_by_faidx
from api.common.genome.blast import outfmt6_read_big, hit_CIP, hit_CALP
from lib.common.os import cmd_run, mkdir, rmdir
from api.xuyuxing.transcriptome.FPKM import get_TPM_matrix, get_FPKM_matrix


def hit_sum_hsp_aln(hit):
    aln_list = []
    for hsp in hit.hsp:
        aln_list.append(hsp.Hsp_align_len)
    return sum(aln_list)


def Denovo2RefCountMap_main(args):

    args.work_dir = os.path.abspath(args.work_dir)

    # build env
    mkdir(args.work_dir, False)

    gene_model_cds_file = args.work_dir + "/gene_model_cds.fna"
    tran_seq_file = args.work_dir + "/trans.fna"
    gene_trans_map_file = args.work_dir + "/gene_trans_map"
    os.symlink(os.path.abspath(args.gene_model_cds_file), gene_model_cds_file)
    os.symlink(os.path.abspath(args.tran_fasta_file), tran_seq_file)
    os.symlink(os.path.abspath(args.gene_trans_map), gene_trans_map_file)

    if args.tran_count_fof:
        tran_count_fof = args.work_dir + "/trans.counts.fof"
        os.symlink(os.path.abspath(args.tran_count_fof), tran_count_fof)
    else:
        tran_count_fof = None

    if args.tran_count_matrix:
        tran_count_matrix = args.work_dir + "/tran_count_matrix"
        os.symlink(os.path.abspath(args.tran_count_matrix), tran_count_matrix)
    else:
        tran_count_matrix = None

    # blast
    cmd_string = "makeblastdb -in %s -dbtype nucl" % gene_model_cds_file
    cmd_run(cmd_string, cwd=args.work_dir)

    bls_out_file = args.work_dir + "/trans_vs_cds.bls"
    cmd_string = "blastn -query %s -db %s -out %s -outfmt 6 -evalue 1e-5 -max_target_seqs 10 -num_threads %d" % (
        tran_seq_file, gene_model_cds_file, bls_out_file, args.threads)
    cmd_run(cmd_string, cwd=args.work_dir)

    # id map
    seq_dict = read_fasta_by_faidx(tran_seq_file)
    len_dict = {i: seq_dict[i].len() for i in seq_dict}

    all_tg2tt_dict = {}
    for tmp_id, tmp_dict in tsv_file_dict_parse_big(gene_trans_map_file, fieldnames=['tg', 'tt'], key_col='tt'):
        tg = tmp_dict['tg']
        all_tg2tt_dict.setdefault(tg, []).append(tmp_dict['tt'])

    tg2tt_dict = {}
    for tg in all_tg2tt_dict:
        tg2tt_dict[tg] = sorted(
            all_tg2tt_dict[tg], key=lambda x: len_dict[x], reverse=True)[0]

    tt2tg_dict = {tg2tt_dict[tg]: tg for tg in tg2tt_dict}

    # read blast
    t2g_dict = {}
    for query in outfmt6_read_big(bls_out_file):
        if query.qID not in tt2tg_dict:
            continue

        q_id = tt2tg_dict[query.qID]

        t2g_dict[q_id] = []
        query.qLen = len_dict[query.qID]
        for hit in query.hit:
            cip = hit_CIP(hit)
            calp = hit_CALP(hit)
            sum_aln = hit_sum_hsp_aln(hit)

            if cip > 0.6 and calp > 0.5 and sum_aln > 50:
                t2g_dict[q_id].append((hit.Hit_id, (cip * calp * sum_aln)))

        if len(t2g_dict[q_id]) != 0:
            t2g_dict[q_id] = sorted(
                t2g_dict[q_id], key=lambda x: x[1], reverse=True)[0][0]
        else:
            del t2g_dict[q_id]

    g2t_dict = {}
    for t in t2g_dict:
        g = t2g_dict[t]
        if g not in g2t_dict:
            g2t_dict[g] = []
        g2t_dict[g].append(t)

    # read tran count
    if tran_count_fof:
        rsem_file_dict = {}
        sample_list = []
        with open(tran_count_fof, 'r') as f:
            for l in f:
                l.strip()
                s, r_file = l.split()
                rsem_file_dict[s] = r_file
                sample_list.append(s)

        tran_count_dict = {}
        for s in rsem_file_dict:
            r_file = rsem_file_dict[s]
            for tmp_id, tmp_dict in tsv_file_dict_parse_big(r_file, key_col='gene_id'):
                tran_count_dict.setdefault(
                    tmp_id, {sd: 0 for sd in sample_list})
                tran_count_dict[tmp_id][s] = round(
                    float(tmp_dict['expected_count']))

    elif tran_count_matrix:
        ttran_count_dict = {}
        for tmp_id, tmp_dict in tsv_file_dict_parse_big(tran_count_matrix, key_col=''):
            sample_list = sorted([i for i in tmp_dict if i != ''])
            ttran_count_dict[tmp_id] = {
                s: round(float(tmp_dict[s])) for s in sample_list}

        tran_count_dict = {}
        for tg in all_tg2tt_dict:
            tran_count_dict[tg] = {s: 0 for s in sample_list}
            for tt in all_tg2tt_dict[tg]:
                if tt in ttran_count_dict:
                    for s in ttran_count_dict[tt]:
                        tran_count_dict[tg][s] += ttran_count_dict[tt][s]

    # map count to gene
    gene_count_dict = {}
    for g in g2t_dict:
        gene_count_dict[g] = {i: 0 for i in sample_list}
        for t in g2t_dict[g]:
            sample_count_dict = tran_count_dict[t]
            for s in sample_count_dict:
                gene_count_dict[g][s] += tran_count_dict[t][s]

    # output
    with open(args.output_prefix+".counts.matrix", 'w') as f:
        f.write("Gene\t" + "\t".join(sample_list) + "\n")

        for g in gene_count_dict:
            c_list = []
            for s in sample_list:
                c = gene_count_dict[g][s]
                c_list.append(str(c))
            "\t".join(c_list)

            f.write(g + "\t" + "\t".join(c_list) + "\n")

    # get TPM
    gene_list = sorted(list(gene_count_dict.keys()))
    count_matrix = []
    for g in gene_list:
        g_r = []
        for s in sample_list:
            g_r.append(gene_count_dict[g][s])
        count_matrix.append(g_r)
    count_matrix = np.array(count_matrix)

    seq_dict = read_fasta_by_faidx(gene_model_cds_file)
    cds_len_list = [seq_dict[i].len() for i in gene_list]

    tpm_matrix = get_TPM_matrix(count_matrix, cds_len_list)

    # output
    with open(args.output_prefix+".tpm.matrix", 'w') as f:
        f.write("Gene\t" + "\t".join(sample_list) + "\n")

        for g in range(len(gene_list)):
            c_list = []
            for s in range(len(sample_list)):
                c = tpm_matrix[g][s]
                c_list.append(str(c))
            "\t".join(c_list)

            g_id = gene_list[g]
            f.write(g_id + "\t" + "\t".join(c_list) + "\n")

    # get FPKM
    fpkm_matrix = get_FPKM_matrix(count_matrix, cds_len_list)

    # output
    with open(args.output_prefix+".fpkm.matrix", 'w') as f:
        f.write("Gene\t" + "\t".join(sample_list) + "\n")

        for g in range(len(gene_list)):
            c_list = []
            for s in range(len(sample_list)):
                c = fpkm_matrix[g][s]
                c_list.append(str(c))
            "\t".join(c_list)

            g_id = gene_list[g]
            f.write(g_id + "\t" + "\t".join(c_list) + "\n")


def Count2TMM_main(args):
    count_matrix, sample_list, gene_list = read_matrix_file(args.count_matrix)

    gene_length_dict = {}
    with open(args.gene_length_file, 'r') as f:
        for i in f:
            i.strip()
            g_id,length = i.split()
            gene_length_dict[g_id] = float(length)

    gene_length_list = [gene_length_dict[i] for i in gene_list]

    fpkm = get_FPKM_matrix(count_matrix, gene_length_list)
    tpm = get_TPM_matrix(count_matrix, gene_length_list)

    # output
    with open(args.output_prefix+".fpkm.matrix", 'w') as f:
        f.write("Gene\t" + "\t".join(sample_list) + "\n")

        for g in range(len(gene_list)):
            c_list = []
            for s in range(len(sample_list)):
                c = fpkm[g][s]
                c_list.append(str(c))
            "\t".join(c_list)

            g_id = gene_list[g]
            f.write(g_id + "\t" + "\t".join(c_list) + "\n")

    # output
    with open(args.output_prefix+".tpm.matrix", 'w') as f:
        f.write("Gene\t" + "\t".join(sample_list) + "\n")

        for g in range(len(gene_list)):
            c_list = []
            for s in range(len(sample_list)):
                c = tpm[g][s]
                c_list.append(str(c))
            "\t".join(c_list)

            g_id = gene_list[g]
            f.write(g_id + "\t" + "\t".join(c_list) + "\n")
                        
    tpm_file = args.output_prefix+".tpm.matrix"
    tmm_file = args.output_prefix+".TMM.matrix"

    cmd_string = "run_TMM_scale_matrix.pl --matrix %s > %s" % (tpm_file, tmm_file)
    cmd_run(cmd_string)

if __name__ == "__main__":

    class abc():
        pass

    args = abc()

    args.gene_model_cds_file = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Striga_asiatica/T4170N0.gene_model.cds.fasta'
    args.tran_count_fof = None
    args.tran_count_matrix = '/lustre/home/xuyuxing/Work/Orobanchaceae/Trans/clean_data/Sas/transcript.counts.matrix'
    args.tran_fasta_file = '/lustre/home/xuyuxing/Work/Orobanchaceae/Trans/clean_data/Sas/Trinity.fasta'
    args.gene_trans_map = '/lustre/home/xuyuxing/Work/Orobanchaceae/Trans/clean_data/Sas/Trinity.fasta.gene_trans_map'
    args.work_dir = '/lustre/home/xuyuxing/Work/Orobanchaceae/Trans/clean_data/Sas/tmp'
    args.output_prefix = '/lustre/home/xuyuxing/Work/Orobanchaceae/Trans/clean_data/Sas/Gene'
    args.threads = 56



import os
from Bio import Phylo
from io import StringIO
import api.xuyuxing.comp_genome.orthofinder_tree_resolve.tree as tree_lib
from api.xuyuxing.comp_genome.orthofinder_tree_resolve.trees2ologs_of import CheckAndRootTree, Resolve
from lib.common.evolution.tree_operate import map_to_method
from config import dlcpar_search_path
from lib.common.os import cmd_run


def phylo2ete(phylo_tree, tree_type='species_tree'):
    """
    species_tree will keep node name
    gene_tree will keep node support
    """
    tmp_handle = StringIO()
    Phylo.write(phylo_tree, tmp_handle, "newick")
    tree_string = tmp_handle.getvalue().strip()

    if tree_type == 'species_tree':
        ete_tree = tree_lib.Tree(tree_string, format=3)
    elif tree_type == 'gene_tree':
        ete_tree = tree_lib.Tree(tree_string, format=0)

    return ete_tree


def ete2phylo(ete_tree, tree_type='species_tree'):
    """
    species_tree will keep node name
    gene_tree will keep node support
    """
    if tree_type == 'species_tree':
        tree_string = ete_tree.write(format=3)
    elif tree_type == 'gene_tree':
        tree_string = ete_tree.write(format=0)

    phylo_tree = Phylo.read(StringIO(tree_string), 'newick')

    return phylo_tree


def root_gene_tree(gene_tree, species_tree_rooted, GeneToSpecies_dict):
    GeneToSpecies = map_to_method(GeneToSpecies_dict)
    species_tree_rooted_ete = phylo2ete(species_tree_rooted, 'species_tree')
    gene_tree_ete = phylo2ete(gene_tree, 'gene_tree')

    rooted_gene_tree_ete, qHaveSupport = CheckAndRootTree(
        gene_tree_ete.write(format=0), species_tree_rooted_ete, GeneToSpecies)
    rooted_gene_tree = ete2phylo(rooted_gene_tree_ete, tree_type='gene_tree')

    return rooted_gene_tree


def resolve_rooted_gene_tree(rooted_gene_tree, GeneToSpecies_dict):
    GeneToSpecies = map_to_method(GeneToSpecies_dict)

    rooted_gene_tree_ete = phylo2ete(rooted_gene_tree, 'gene_tree')
    resolved_gene_tree_ete = Resolve(rooted_gene_tree_ete, GeneToSpecies)
    resolved_gene_tree = ete2phylo(
        resolved_gene_tree_ete, tree_type='gene_tree')

    return resolved_gene_tree


if __name__ == '__main__':
    # let gene tree resolved

    gene_tree_file = '/lustre/home/xuyuxing/Work/Gel/synteny/20210311/orthofinder/pt_file/OrthoFinder/Results_Jun23/MultipleSequenceAlignments/test/OG0001554.fa.phb'
    gene_tree = Phylo.read(gene_tree_file, 'newick')

    sp_tree_file = '/lustre/home/xuyuxing/Work/Gel/synteny/20210311/orthofinder/pt_file/OrthoFinder/Results_Jun23/Species_Tree/SpeciesTree_rooted_node_labels.txt'
    species_tree_rooted = Phylo.read(sp_tree_file, 'newick')

    GeneToSpecies_dict = {}
    for i in gene_tree.get_terminals():
        GeneToSpecies_dict[i.name] = i.name.split("_")[0]

    # root gene tree by species tree
    rooted_gene_tree = root_gene_tree(
        gene_tree, species_tree_rooted, GeneToSpecies_dict)

    Phylo.draw_ascii(gene_tree)
    Phylo.draw_ascii(rooted_gene_tree)

    # resolve gene tree (orthofinder way)
    resolved_gene_tree = resolve_rooted_gene_tree(
        rooted_gene_tree, GeneToSpecies_dict)

    Phylo.draw_ascii(resolved_gene_tree)

    #



outfmt6_fieldnames = ["query_id", "subject_id", "identity", "alignment_length", "mismatches", "gap_openings",
                        "q_start", "q_end", "s_start", "s_end", "e_value", "bit_score"]


if __name__ == '__main__':
    import argparse

    # argument parse
    parser = argparse.ArgumentParser(
        prog='FormatTools',
    )

    subparsers = parser.add_subparsers(
        title='subcommands', dest="subcommand_name")

    # argparse for outfmt5To6
    parser_a = subparsers.add_parser('outfmt5To6',
                                     help='convert blast results from outfmt 5 to 6',
                                     description='convert blast results from outfmt 5 to 6')

    parser_a.add_argument('input_file', type=str,
                          help='input file with outfmt 5')
    parser_a.add_argument('output_file', type=str, help='output file path')

    # argparse for outfmt5complete
    parser_a = subparsers.add_parser('outfmt5complete',
                                     help='check if outfmt5 is complete')

    parser_a.add_argument('input_file', type=str,
                          help='input file with outfmt 5')

    # argparse for genblasta2BED
    parser_a = subparsers.add_parser('genblasta2BED',
                                     help='convert genblasta output to bed file')

    parser_a.add_argument('input_file', type=str,
                          help='input file with outfmt 5')
    parser_a.add_argument('output_file', type=str, help='output file path')
    parser_a.add_argument('-p', "--ID_prefix", type=str, default='subject_',
                          help='gene output name prefix defaults: subject_')

    # argparse for blast2DB
    parser_a = subparsers.add_parser('blast2DB',
                                     help='save blast results into sqlite db')

    parser_a.add_argument('input_bls', type=str,
                          help='input file with outfmt 6')
    parser_a.add_argument('db_fasta', type=str, help='output database file')
    parser_a.add_argument('-g', "--gzip_flag",
                          help='if bls is gzipped', action='store_true')

    # argparse for outfmt6ToFasta
    parser_a = subparsers.add_parser('outfmt6ToFasta',
                                     help='extract subject sequence by blast outfmt6 results')

    parser_a.add_argument('outfmt6', type=str, help='input file with outfmt 6')
    parser_a.add_argument('db_fasta', type=str,
                          help='input file with database fasta file')
    parser_a.add_argument('output_fasta', type=str, help='output file path')

    # argparse for TrinotateOK
    parser_a = subparsers.add_parser('TrinotateOK',
                                     help='test if a trinotate pipeline is good')

    parser_a.add_argument('transcripts_file', type=str,
                          help='path of transcripts file')
    parser_a.add_argument('-o', "--output_file",
                          type=str, help='output file path')

    # argparse for MD5Checker
    parser_a = subparsers.add_parser('MD5Checker',
                                     help='check md5 files in whole dir')

    parser_a.add_argument('dir_path', type=str, help='path of dir to check')

    # argparse for Count2FPKM
    parser_a = subparsers.add_parser('Count2FPKM',
                                     help='convert featurecount to fpkm')

    parser_a.add_argument('count_file', type=str, help='path of count.txt file')
    parser_a.add_argument('count_summary', type=str, help='path of count.txt.summary file')
    parser_a.add_argument('output_file', type=str, help='path of output file')



    args = parser.parse_args()
    args_dict = vars(args)

    # ---------------------------------------------------------
    # command detail

    # outfmt5to6
    if args_dict["subcommand_name"] == "outfmt5To6":

        from lib.common.util import printer_list
        from api.common.genome.blast import outfmt5_read_big, keep_outfmt6_info

        input_file = args.input_file
        output_file = args.output_file

        output_dict = outfmt5_read_big(input_file, False)
        with open(output_file, 'w') as f:
            for query in output_dict:
                for hsp in keep_outfmt6_info(query):
                    f.write(printer_list(hsp) + "\n")

    elif args_dict["subcommand_name"] == "outfmt5complete":
        from api.common.genome.blast import outfmt5_complete

        if outfmt5_complete(args.input_file):
            print("%s is complete" % args.input_file)
        else:
            print("%s is not complete" % args.input_file)

    elif args_dict["subcommand_name"] == "genblasta2BED":
        import re
        from lib.common.fileIO import tsv_file_dict_parse
        """
        class abc(object):
            pass

        args = abc()

        args.input_file = '/lustre/home/xuyuxing/Work/Csp/ITS/Cau.rRNA'
        args.output_file = '/lustre/home/xuyuxing/Work/Csp/ITS/Cau.rRNA.bed'
        args.ID_prefix = 'Cau_ITS_'
        """

        def fancy_name_parse(input_string):
            contig_name, c_start, c_end = re.search(
                r'^(\S+):(\d+)\.\.(\d+)$', input_string).groups()
            return contig_name, int(c_start), int(c_end)

        gb_file = tsv_file_dict_parse(args.input_file, seq="|",
                                      fieldnames=['query_name', 'subject_name', 'strand', 'gene_cover', 'score',
                                                  'rank'])

        with open(args.output_file, 'w') as f:
            num = 0
            for i in gb_file:
                if gb_file[i]['rank'] is not None and re.match(r'rank:\d+', gb_file[i]['rank']):
                    num = num + 1

                    score = float(
                        re.search(r'score:(.*)', gb_file[i]['score']).group(1))

                    contig_name, c_start, c_end = fancy_name_parse(
                        gb_file[i]['subject_name'])

                    f.write("%s\t%d\t%d\t%s\t%f\t%s\n" % (
                        contig_name, c_start, c_end, args.ID_prefix + str(num), score, gb_file[i]['strand']))

    elif args_dict["subcommand_name"] == "outfmt6ToFasta":
        from lib.common.fileIO import tsv_file_dict_parse
        from pyfaidx import Fasta
        from Bio.Seq import Seq
        from Bio.SeqRecord import SeqRecord

        """
        class abc(object):
            pass

        args = abc()

        args.outfmt6 = '/lustre/home/xuyuxing/Work/Csp/Cleistogrammica/Cau.ITS.bls'
        args.db_fasta = '/lustre/home/xuyuxing/Work/Csp/ITS/Cuscuta.genome.v1.1.fasta'
        args.output_fasta = '/lustre/home/xuyuxing/Work/Csp/Cleistogrammica/Cau.ITS.seq'
        """

        blast_file = tsv_file_dict_parse(
            args.outfmt6, fieldnames=outfmt6_fieldnames)

        ref_dict = Fasta(args.db_fasta)

        with open(args.output_fasta, 'w') as f:
            for ID in blast_file:
                s_name = blast_file[ID]['subject_id']
                s_start = int(blast_file[ID]['s_start'])
                s_end = int(blast_file[ID]['s_end'])

                if s_end > s_start:
                    neg_strand = False
                    strand = "+"
                else:
                    neg_strand = True
                    strand = "-"
                    tmp = s_start
                    s_start = s_end
                    s_end = tmp

                a = ref_dict.get_seq(s_name, s_start, s_end, rc=neg_strand)
                fancy_name = "%s:%d-%d:%s" % (s_name, s_start, s_end, strand)

                contig_record = SeqRecord(
                    Seq(a.seq), id=ID, description=fancy_name)

                f.write(contig_record.format("fasta"))

    elif args_dict["subcommand_name"] == "TrinotateOK":
        import re
        import os
        from BCBio import GFF
        from lib.common.fileIO import tsv_file_dict_parse
        from pyfaidx import Fasta
        """
        class abc(object):
            pass

        args = abc()

        args.transcripts_file = '/lustre/home/xuyuxing/Database/1kp/annotation/ERS631159/ERS631159.fasta'
        """
        work_dir = os.path.dirname(args.transcripts_file)

        check_list_dir = {
            "TransDecoder": {
                "bed_file": "uncheck",
                "cds_file": "uncheck",
                "gff3_file": "uncheck",
                "pep_file": "uncheck"
            },
            "blast": {
                "blastx": "uncheck",
                "blastp": "uncheck"
            },
            "pfam": {
                "pfam_log": "uncheck",
                "pfam_tab": "uncheck"
            },
            "signalp": {
                "signalp": "uncheck"
            },

            "tmhmm": {
                "tmhmm": "uncheck"
            },
            "rnammer": {
                "rnammer_out": "uncheck",
                "rnammer_gff": "uncheck",
            },
            "trinotate": {
                "trinotate_tab": "uncheck",
                "trinotate_go": "uncheck",
            }
        }

        # TransDecoder
        transdecoder_files = {
            "bed_file": args.transcripts_file + ".transdecoder.bed",
            "cds_file": args.transcripts_file + ".transdecoder.cds",
            "gff3_file": args.transcripts_file + ".transdecoder.gff3",
            "pep_file": args.transcripts_file + ".transdecoder.pep"
        }

        for i in transdecoder_files:
            file_name = transdecoder_files[i]
            if os.path.exists(file_name):
                if i == 'bed_file':
                    tran_id_list = []
                    with open(file_name, 'r') as f:
                        for each_line in f:
                            if re.match(r'^track name=', each_line):
                                continue
                            each_line = re.sub('\n', '', each_line)
                            tran_id = each_line.split('\t')[0]
                            tran_id_list.append(tran_id)
                    check_list_dir["TransDecoder"][i] = len(
                        list(set(tran_id_list)))
                elif i == "gff3_file":
                    num = 0
                    with open(file_name, 'r') as f:
                        for rec in GFF.parse(f):
                            num = num + 1
                    check_list_dir["TransDecoder"][i] = num
                else:
                    check_list_dir["TransDecoder"][i] = len(
                        Fasta(file_name).keys())
            else:
                check_list_dir["TransDecoder"][i] = 'failed'

        # Blast/diamond
        blast_files = {
            "blastx": work_dir + "/swissprot.blastx.outfmt6",
            "blastp": work_dir + "/swissprot.blastp.outfmt6"
        }

        for i in blast_files:
            file_name = blast_files[i]
            if os.path.exists(file_name):
                tran_id_list = []
                with open(file_name, 'r') as f:
                    for each_line in f:
                        each_line = re.sub('\n', '', each_line)
                        tran_id = each_line.split('\t')[0]
                        tran_id_list.append(tran_id)
                check_list_dir["blast"][i] = len(list(set(tran_id_list)))
            else:
                check_list_dir["blast"][i] = 'failed'

        # pfam
        pfam_files = {
            "pfam_log": work_dir + "/pfam.log",
            "pfam_tab": work_dir + "/TrinotatePFAM.out"
        }

        for i in pfam_files:
            file_name = pfam_files[i]
            if os.path.exists(file_name):
                if i == "pfam_log":
                    tran_id_list = []
                    with open(file_name, 'r') as f:
                        for each_line in f:
                            each_line = re.sub('\n', '', each_line)
                            match_list = re.findall(
                                r'^Query:\s+(\S+)\s+\S+$', each_line)
                            if len(match_list) != 0:
                                tran_id = match_list[0]
                                tran_id_list.append(tran_id)
                    check_list_dir["pfam"][i] = len(list(set(tran_id_list)))
                elif i == "pfam_tab":
                    tran_id_list = []
                    with open(file_name, 'r') as f:
                        for each_line in f:
                            if re.match(r'^#', each_line):
                                continue
                            each_line = re.sub('\n', '', each_line)
                            tran_id = each_line.split()[3]
                            tran_id_list.append(tran_id)
                    check_list_dir["pfam"][i] = len(list(set(tran_id_list)))
            else:
                check_list_dir["pfam"][i] = 'failed'

        # SIGNALP
        signalp_files = {
            "signalp": work_dir + "/signalp.out_summary.signalp5"
        }

        for i in signalp_files:
            file_name = signalp_files[i]
            if os.path.exists(file_name):
                tran_id_list = []
                with open(file_name, 'r') as f:
                    for each_line in f:
                        if re.match(r'^#', each_line):
                            continue
                        each_line = re.sub('\n', '', each_line)
                        tran_id = each_line.split()[0]
                        tran_id_list.append(tran_id)
                check_list_dir["signalp"][i] = len(list(set(tran_id_list)))
            else:
                check_list_dir["signalp"][i] = 'failed'

        # TMHMM
        tmhmm_files = {
            "tmhmm": work_dir + "/tmhmm.out"
        }

        for i in tmhmm_files:
            file_name = tmhmm_files[i]
            if os.path.exists(file_name):
                tran_id_list = []
                with open(file_name, 'r') as f:
                    for each_line in f:
                        if re.match(r'^#', each_line):
                            continue
                        each_line = re.sub('\n', '', each_line)
                        tran_id = each_line.split()[0]
                        tran_id_list.append(tran_id)
                check_list_dir["tmhmm"][i] = len(list(set(tran_id_list)))
            else:
                check_list_dir["tmhmm"][i] = 'failed'

        # RNAMMER
        rnammer_files = {
            "rnammer_out": work_dir + "/tmp.superscaff.rnammer.gff",
            "rnammer_gff": args.transcripts_file + ".rnammer.gff",
        }

        for i in rnammer_files:
            file_name = rnammer_files[i]
            if os.path.exists(file_name):
                tran_id_list = []
                num = 0
                with open(file_name, 'r') as f:
                    for rec in GFF.parse(f):
                        for gene in rec.features:
                            num = num + 1
                check_list_dir["rnammer"][i] = num
            else:
                check_list_dir["rnammer"][i] = 'failed'

        # TRINOTATE report
        trinotate_files = {
            "trinotate_tab": work_dir + "/Trinotate.xls",
            "trinotate_go": work_dir + "/Trinotate.xls.gene_ontology"
        }

        col_name = ['sprot_Top_BLASTX_hit',
                    'RNAMMER',
                    'prot_id',
                    'prot_coords',
                    'sprot_Top_BLASTP_hit',
                    'Pfam',
                    'SignalP',
                    'TmHMM',
                    'eggnog',
                    'Kegg',
                    'gene_ontology_blast',
                    'gene_ontology_pfam',
                    'transcript',
                    'peptide']

        for i in trinotate_files:
            file_name = trinotate_files[i]
            if os.path.exists(file_name):
                if i == "trinotate_tab":

                    col_hit_num_dir = {}

                    file_dict = tsv_file_dict_parse(file_name)

                    for j in col_name:
                        gene_num = len(
                            list(set([file_dict[k]['#gene_id'] for k in file_dict if file_dict[k][j] != '.'])))
                        transcript_num = len(
                            list(set([file_dict[k]['transcript_id'] for k in file_dict if file_dict[k][j] != '.'])))
                        col_hit_num_dir[j] = (gene_num, transcript_num)

                    check_list_dir["trinotate"][i] = col_hit_num_dir
                elif i == "trinotate_go":
                    tran_id_list = []
                    with open(file_name, 'r') as f:
                        for each_line in f:
                            if re.match(r'^#', each_line):
                                continue
                            each_line = re.sub('\n', '', each_line)
                            tran_id = each_line.split()[0]
                            tran_id_list.append(tran_id)
                    check_list_dir["trinotate"][i] = len(tran_id_list)
            else:
                check_list_dir["trinotate"][i] = 'failed'

        if args.output_file is not None:
            import json

            with open(args.output_file, 'w') as f:
                json.dump(check_list_dir, f)

        # make output
        if check_list_dir["trinotate"]["trinotate_tab"] == 'failed':
            print("FAILED: complete failed")
        else:
            num = 0
            for i in col_name:
                if check_list_dir["trinotate"]["trinotate_tab"][i] == (0, 0):
                    num = num + 1
            if num > 4:
                print("FAILED: too many field failed")
            else:
                print("OK: with %d bad" % num)

    elif args_dict["subcommand_name"] == "MD5Checker":
        import os
        from lib.common.os import cmd_run


        def check_md5(dir_path):
            dir_path = os.path.abspath(dir_path)

            file_dir_list = os.listdir(dir_path)
            for tmp_name in file_dir_list:
                tmp_name_full_path = dir_path + "/" + tmp_name
                if os.path.isdir(tmp_name_full_path):
                    check_md5(tmp_name_full_path)
                else:
                    tmp_list = tmp_name.split("_")
                    if len(tmp_list) > 1:
                        if tmp_list[0] == 'MD5':
                            cmd_string = "md5sum -c %s" % tmp_name
                            # print(dir_path)
                            # print(cmd_string)
                            flag, output, error = cmd_run(cmd_string, cwd=dir_path, retry_max=5, silence=True,
                                                          log_file=None)
                            print(output)

        check_md5(args.dir_path)

    elif args_dict["subcommand_name"] == "blast2DB":
        from api.common.genome.blast import blast_to_sqlite

        blast_to_sqlite(args.db_fasta, args.input_bls, None,
                        None, 6, None, None, None, False, args.gzip_flag)

    elif args_dict["subcommand_name"] == "Count2FPKM":
        """
        class abc(object):
            pass

        args = abc()
        args.count_file = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Gastrodia_elata/transcriptome/count.txt'
        args.count_summary = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Gastrodia_elata/transcriptome/count.txt.summary'
        args.gff_file = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Gastrodia_elata/transcriptome/Gel.gene_pseudogene.gff3'
        args.output_file = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Gastrodia_elata/transcriptome/FPKM.txt'
        """

        from lib.common.fileIO import tsv_file_dict_parse
        from lib.common.util import printer_list
        from lib.common.genome.genome_feature2 import read_gff_file, Gene


        def fpkm(count,length,mapped_count):
            return count/((length/1000)*(mapped_count/1000000))

        count_dict = tsv_file_dict_parse(args.count_file, seq='\t', ignore_head_num=1)
        count_summary_dict = tsv_file_dict_parse(args.count_summary, seq='\t')

        mapped_count = {}
        for i in count_summary_dict['ID_0']:
            if i != 'Status':
                mapped_count[i] = int(count_summary_dict['ID_0'][i])

        gff_dict = read_gff_file(args.gff_file)

        gene_length_dict = {}
        for i in gff_dict:
            for j in gff_dict[i]:
                gene_tmp = Gene(from_gf=gff_dict[i][j])
                gene_max_length = 0
                for mRNA_tmp in gene_tmp.sub_features:
                    mRNA_tmp.sgf_len()
                    if 'exon' in mRNA_tmp.sgf_len_dir:
                        gene_length = mRNA_tmp.sgf_len_dir['exon']
                    else:
                        gene_length = mRNA_tmp.sgf_len_dir['CDS']
                    if gene_length > gene_max_length:
                        gene_max_length = gene_length
                gene_length_dict[j] = gene_max_length
                
        with open(args.output_file, 'w') as f:
            header = list(count_dict['ID_0'].keys())
            sample_list = header[6:]
            g_info_list = header[:6]

            f.write(printer_list(header)+"\n")

            for i in count_dict:
                data_info = count_dict[i]
                g_id = data_info['Geneid']
                g_len = gene_length_dict[g_id]
                data_info['Length'] = g_len

                fpkm_list = [fpkm(int(data_info[i]),g_len,mapped_count[i]) for i in sample_list]

                output_list = [data_info[i] for i in g_info_list] + fpkm_list

                f.write(printer_list(output_list)+"\n")

    elif args_dict["subcommand_name"] == "id_replace":
        import re
        import sys

        target_file = sys.argv[1]
        map_file = sys.argv[2]
        output_file = sys.argv[3]

        replace_dict = {}
        with open(map_file, 'r') as f:
            for each_line in f:
                each_line.strip()
                a,b = each_line.split()
                replace_dict[a] = b

        with open(output_file, 'w') as fo:
            with open(target_file, 'r') as f:
                for each_line in f:
                    each_line.strip()
                    get_list = list(set(re.findall(r'[a-zA-Z]+_\d+', each_line)))
                    num = 0
                    
                    from_string = None
                    for i in get_list:
                        if i in replace_dict:
                            num += 1
                            from_string = i

                    if num > 1:
                        raise ValueError('two more match %s' % each_line)
                    elif num == 1:
                        to_string = replace_dict[from_string]
                        each_line = each_line.replace(from_string, to_string)

                    fo.write(each_line)



